using System;
using System.Collections.Generic;
using System.ComponentModel;using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.ServiceModel;
using System.Configuration;
using System.Reflection;
using Callista.BuildLab.Core;
using Callista.BuildLab.Core.Model;
using System.Security.Principal;

namespace Callista.BuildLab.AdminClient
{
    public partial class MainForm : Form, IEventNotificationCallback
    {
        public MainForm()
        {
            InitializeComponent();
        }

        private void DisplayError(FaultException ex)
        {
            MessageBox.Show(ex.ToString(),ex.Reason.ToString());
        }

        #region Private Members
        
        private bool _isInitializing = true;
        private bool _fullScreenMode = false;
        private int _tabPositionWhenEnteringFullScreenMode = 0;
        private List<TabPage> _hiddenTabs = new List<TabPage>();
        private NotifyIcon trayIcon;
        private int _colSortedIndexBuildLog;
        private ListSortDirection _colSortOrderBuildLog;
        private int _colSortedIndexBuildEventLog;
        private ListSortDirection _colSortOrderBuildEventLog;
        private int _colSortedIndexBuildQueue;
        private ListSortDirection _colSortOrderBuildQueue;

        private ContextMenu contextMenuBuildTypeInfo = new System.Windows.Forms.ContextMenu();
        private MenuItem menuItemRefreshBuildTypeInfo = new System.Windows.Forms.MenuItem();
        private MenuItem menuItemCopyBuildTypeInfo = new System.Windows.Forms.MenuItem();

        public delegate void InvokeDelegate();

        #endregion

        #region Form Initialization

        private void PopulateTeamProjects(List<TeamProject> projectList, ComboBox target)
        {
            target.SuspendLayout();
            target.DataSource = projectList;
            target.DisplayMember = "Name";
            target.SelectedIndex = 0;
            target.ResumeLayout();
        }

        private void PopulateTeamProjects(List<TeamProject> projectList, TreeView target)
        {
            target.Nodes.Clear();
            foreach (TeamProject p in projectList)
            {
                if (p.Name != string.Empty)
                {
                    TreeNode projectNode = target.Nodes.Add(p.Name);
                    foreach (BuildType bt in p.BuildTypes.Values)
                    {
                        TreeNode buildTypeNode = projectNode.Nodes.Add(bt.Name);
                    }
                }
            }
        }

        private void PopulateBuildTypes(TeamProject project, ComboBox target)
        {
            IBuildTypeAdministration proxy = null;
            try
            {
                proxy = CreateProxy<IBuildTypeAdministration>();

                List<BuildType> teamBuildTypes = proxy.ListBuildTypes(project.Name);
                teamBuildTypes.Insert(0, new BuildType("", ""));

                target.SuspendLayout();
                target.DataSource = teamBuildTypes;
                target.DisplayMember = "Name";
                target.SelectedIndex = 0;
                target.ResumeLayout();
            }
            catch (FaultException ex)
            {
                DisplayError(ex);
            }
            finally
            {
                CloseProxy((ICommunicationObject)proxy);
            }
        }

        private void InitializeControls()
        {
            List<TeamProject> teamProjects = null;
            IBuildTypeAdministration proxy = null;
            try
            {
                proxy = CreateProxy<IBuildTypeAdministration>();
                teamProjects = proxy.ListTeamProjects(); 
            }
            catch (FaultException ex)
            {
                DisplayError(ex);
            }
            finally
            {
                CloseProxy((ICommunicationObject)proxy);
            }

            PopulateTeamProjects(teamProjects, BuildTypeTreeView);
            
            teamProjects.Insert(0, new TeamProject(""));

            PopulateTeamProjects(teamProjects, BQTeamProject);
            PopulateTeamProjects(teamProjects, BLTeamProject);
            PopulateTeamProjects(teamProjects, BHTeamProject);

            BLBuildResult.Items.Add("");
            BLBuildResult.Items.Add(BuildStatus.BuildFailed);
            BLBuildResult.Items.Add(BuildStatus.BuildRunning);
            BLBuildResult.Items.Add(BuildStatus.BuildStopped);
            BLBuildResult.Items.Add(BuildStatus.BuildSucceeded);

            BHEventType.Items.Add("");
            BHEventType.Items.Add(EventType.QueueCheckin);
            BHEventType.Items.Add(EventType.StartCheckin);
            BHEventType.Items.Add(EventType.RemoveCheckin);
            BHEventType.Items.Add(EventType.UndeliverableCheckin);
            BHEventType.Items.Add(EventType.CompletedCheckin);
            BHEventType.Items.Add(EventType.FailedCheckin);
            BHEventType.Items.Add(EventType.QueueScheduled);
            BHEventType.Items.Add(EventType.StartScheduled);
            BHEventType.Items.Add(EventType.RemoveScheduled);
            BHEventType.Items.Add(EventType.UndeliverableScheduled);
            BHEventType.Items.Add(EventType.CompletedScheduled);
            BHEventType.Items.Add(EventType.FailedScheduled);
            BHEventType.Items.Add(EventType.Delete);
            BHEventType.Items.Add(EventType.Error);

            BHStartDTM.Value = BHStartDTM.Value.AddDays(-7);
            BHEndDTM.Value = DateTime.Now;

            BuildQueueGrid.Columns.Add("ID", "QueueNumber");
            BuildQueueGrid.Columns.Add("TeamProject", "TeamProject");
            BuildQueueGrid.Columns.Add("BuildType", "BuildType");
            BuildQueueGrid.Columns.Add("TriggeringUser", "TriggeringUser");
            BuildQueueGrid.Columns.Add("TriggeringChangesetID", "TriggeringChangeset");
            BuildQueueGrid.Columns["ID"].DataPropertyName = "ID";
            BuildQueueGrid.Columns["ID"].AutoSizeMode = DataGridViewAutoSizeColumnMode.ColumnHeader;
            BuildQueueGrid.Columns["TeamProject"].DataPropertyName = "TeamProjectName";
            BuildQueueGrid.Columns["BuildType"].DataPropertyName = "BuildTypeName";
            BuildQueueGrid.Columns["TriggeringUser"].DataPropertyName = "TriggeringUser";
            BuildQueueGrid.Columns["TriggeringChangesetID"].DataPropertyName = "TriggeringChangesetID";
            BuildQueueGrid.Columns["TriggeringChangesetID"].AutoSizeMode = DataGridViewAutoSizeColumnMode.ColumnHeader;
            BuildQueueGrid.AutoGenerateColumns = false;
            BuildQueueGrid.ClearSelection();
            BuildQueueGrid.ScrollBars = ScrollBars.Both;

            _colSortedIndexBuildQueue  = 0;
            _colSortOrderBuildQueue = ListSortDirection.Ascending;

            BuildLogGrid.Columns.Add("BuildUri", "BuildUri");
            BuildLogGrid.Columns.Add("BuildNumber", "BuildNumber");
            BuildLogGrid.Columns.Add("BuildStatus", "BuildStatus");
            BuildLogGrid.Columns.Add("BuildType", "BuildType");
            BuildLogGrid.Columns.Add("BuildQuality", "BuildQuality");
            BuildLogGrid.Columns.Add("RequestedBy", "RequestedBy");
            BuildLogGrid.Columns.Add("StartTime", "StartTime");
            BuildLogGrid.Columns.Add("FinishTime", "FinishTime");
            BuildLogGrid.Columns["BuildUri"].DataPropertyName = "BuildUri";
            BuildLogGrid.Columns["BuildUri"].Visible = false;
            BuildLogGrid.Columns["BuildNumber"].DataPropertyName = "BuildNumber";
            BuildLogGrid.Columns["BuildStatus"].DataPropertyName = "Status";
            BuildLogGrid.Columns["BuildType"].DataPropertyName = "BuildType";
            BuildLogGrid.Columns["BuildQuality"].DataPropertyName = "Quality";
            BuildLogGrid.Columns["RequestedBy"].DataPropertyName = "RequestedBy";
            BuildLogGrid.Columns["StartTime"].DataPropertyName = "StartTime";
            BuildLogGrid.Columns["FinishTime"].DataPropertyName = "FinishTime";
            BuildLogGrid.AutoGenerateColumns = false;
            BuildLogGrid.ClearSelection();
            BuildLogGrid.ScrollBars = ScrollBars.Both;

            _colSortedIndexBuildLog = 6;
            _colSortOrderBuildLog = ListSortDirection.Descending;

            BuildEventLog.Columns.Add("EventTime", "EventTime");
            BuildEventLog.Columns.Add("EventType", "EventType");
            BuildEventLog.Columns.Add("TeamProject", "TeamProject");
            BuildEventLog.Columns.Add("BuildType", "BuildType");
            BuildEventLog.Columns.Add("BuildUri", "BuildUri");
            BuildEventLog.Columns.Add("TriggeredBy", "TriggeredBy");
            BuildEventLog.Columns.Add("Message", "Message");
            BuildEventLog.Columns["EventTime"].DataPropertyName = "EventTime";
            BuildEventLog.Columns["EventType"].DataPropertyName = "EventType";
            BuildEventLog.Columns["TeamProject"].DataPropertyName = "TeamProject";
            BuildEventLog.Columns["BuildType"].DataPropertyName = "BuildType";
            BuildEventLog.Columns["BuildUri"].DataPropertyName = "BuildUri";
            BuildEventLog.Columns["TriggeredBy"].DataPropertyName = "TriggeredBy";
            BuildEventLog.Columns["Message"].DataPropertyName = "Message";
            BuildEventLog.AutoGenerateColumns = false;
            BuildEventLog.ClearSelection();
            BuildEventLog.ScrollBars = ScrollBars.Both;

            _colSortedIndexBuildEventLog = 0;
            _colSortOrderBuildEventLog = ListSortDirection.Descending;

            RetentionPolicyGridView.Columns.Add("ID", "ID");
            RetentionPolicyGridView.Columns.Add("BuildStatus", "BuildStatus");
            RetentionPolicyGridView.Columns.Add("BuildQuality", "BuildQuality");
            RetentionPolicyGridView.Columns.Add("Type", "RetentionType");
            RetentionPolicyGridView.Columns.Add("RetentionQuantity", "RetentionQuantity");
            RetentionPolicyGridView.Columns["ID"].DataPropertyName = "ID";
            RetentionPolicyGridView.Columns["ID"].Visible = false;
            RetentionPolicyGridView.Columns["BuildStatus"].DataPropertyName = "BuildStatus";
            RetentionPolicyGridView.Columns["BuildQuality"].DataPropertyName = "BuildQuality";
            RetentionPolicyGridView.Columns["Type"].DataPropertyName = "Type";
            RetentionPolicyGridView.Columns["RetentionQuantity"].DataPropertyName = "RetentionQuantity";
            RetentionPolicyGridView.AutoGenerateColumns = false;
            RetentionPolicyGridView.ClearSelection();

            TriggersGridView.Columns.Add("ID", "ID");
            TriggersGridView.Columns.Add("Type", "TriggerType");
            TriggersGridView.Columns.Add("Data", "TriggerPath");
            TriggersGridView.Columns["ID"].DataPropertyName = "ID";
            TriggersGridView.Columns["ID"].Visible = false;
            TriggersGridView.Columns["Type"].DataPropertyName = "Type";
            TriggersGridView.Columns["Data"].DataPropertyName = "Data";
            TriggersGridView.AutoGenerateColumns = false;
            TriggersGridView.ClearSelection();

            System.Windows.Forms.ContextMenu contextMenuBuildQueueGrid = new System.Windows.Forms.ContextMenu();
            System.Windows.Forms.MenuItem menuItemRefreshBuildQueueGrid = new System.Windows.Forms.MenuItem();

            contextMenuBuildQueueGrid.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] { menuItemRefreshBuildQueueGrid });
            menuItemRefreshBuildQueueGrid.Index = 0;
            menuItemRefreshBuildQueueGrid.Text = "Refresh";
            menuItemRefreshBuildQueueGrid.Click += new System.EventHandler(refreshBuildQueueGridContextMenuItem_Click);

            System.Windows.Forms.ContextMenu contextMenuBuildLogGrid = new System.Windows.Forms.ContextMenu();
            System.Windows.Forms.MenuItem menuItemRefreshBuildLogGrid = new System.Windows.Forms.MenuItem();

            contextMenuBuildLogGrid.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] { menuItemRefreshBuildLogGrid });
            menuItemRefreshBuildLogGrid.Index = 0;
            menuItemRefreshBuildLogGrid.Text = "Refresh";
            menuItemRefreshBuildLogGrid.Click += new System.EventHandler(refreshBuildLogGridContextMenuItem_Click);

            System.Windows.Forms.ContextMenu contextMenuBuildEventsGrid = new System.Windows.Forms.ContextMenu();
            System.Windows.Forms.MenuItem menuItemRefreshBuildEventsGrid = new System.Windows.Forms.MenuItem();

            contextMenuBuildEventsGrid.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] { menuItemRefreshBuildEventsGrid });
            menuItemRefreshBuildEventsGrid.Index = 0;
            menuItemRefreshBuildEventsGrid.Text = "Refresh";
            menuItemRefreshBuildEventsGrid.Click += new System.EventHandler(refreshBuildEventsGridContextMenuItem_Click);

            contextMenuBuildTypeInfo = new System.Windows.Forms.ContextMenu();
            menuItemCopyBuildTypeInfo = new System.Windows.Forms.MenuItem();
            menuItemRefreshBuildTypeInfo = new System.Windows.Forms.MenuItem();

            contextMenuBuildTypeInfo.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] { menuItemCopyBuildTypeInfo, menuItemRefreshBuildTypeInfo });
            contextMenuBuildTypeInfo.Popup += new EventHandler(contextMenuBuildTypeInfo_Popup);
            menuItemCopyBuildTypeInfo.Index = 0;
            menuItemCopyBuildTypeInfo.Text = "Copy";
            menuItemCopyBuildTypeInfo.Click += new EventHandler(copyBuildTypeInfoContextMenuItem_Click);
            menuItemRefreshBuildTypeInfo.Index = 1;
            menuItemRefreshBuildTypeInfo.Text = "Refresh";
            menuItemRefreshBuildTypeInfo.Click += new EventHandler(refreshBuildTypeInfoContextMenuItem_Click);

            mainTabControl.TabPages[1].ContextMenu = contextMenuBuildQueueGrid;
            mainTabControl.TabPages[2].ContextMenu = contextMenuBuildLogGrid;
            mainTabControl.TabPages[3].ContextMenu = contextMenuBuildEventsGrid;
            mainTabControl.TabPages[4].ContextMenu = contextMenuBuildTypeInfo;

            trayIcon = new NotifyIcon();
            trayIcon.ContextMenu = new ContextMenu();
            trayIcon.ContextMenu.MenuItems.AddRange(new MenuItem[] { new MenuItem("&Exit",new System.EventHandler(this.TrayIcon_ContextMenuItemExitClick)) });
            trayIcon.DoubleClick += new EventHandler(TrayIcon_DoubleClick);
            trayIcon.Icon = new Icon(typeof(Program), "App.ico");
            trayIcon.Text = "TfsBuildLab Administration";
            trayIcon.Visible = false;

            _isInitializing = false;

            reportsComboBox.Items.AddRange(ConfigurationManager.AppSettings["ReportList"].Split(';'));
            string initialReport = ConfigurationManager.AppSettings["InitalReport"];
            if (!string.IsNullOrEmpty(initialReport))
            {
                reportsComboBox.SelectedItem = initialReport;
            }

        }

        #endregion

        #region Form Menu

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            using (new WaitCursor(this))
            {
                this.Close();
            }
        }

        private void resetAllCachesToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            using (new WaitCursor(this))
            {
                _adminProxy.ForceRecaching();
            }
        }

        private void tFSBuildLabCodeplexToolStripMenuItem_Click(object sender, EventArgs e)
        {
            System.Diagnostics.Process.Start("http://www.codeplex.com/tfsbuildlab/");
        }

        private void aboutTFSBuildLabToolStripMenuItem_Click(object sender, EventArgs e)
        {
            AboutBox about = new AboutBox();
            about.ShowDialog(this);
        }

        #endregion

        #region Form Events

        private void MainForm_Load(object sender, EventArgs e)
        {
            using (new WaitCursor(this))
            {
                SplashForm splash = new SplashForm();
                splash.Show();
                splash.InfoText = "Connecting to server...";
                if (ConnectToServer())
                {
                    splash.InfoText = "Initializing UI...";
                    InitializeControls();
                    splash.Close();
                }

                this.Text = this.Text + string.Format(" [{0}]", TfsBuildLabServer);
            }
        }

        protected override void OnClosed(EventArgs e)
        {
            using (new WaitCursor(this))
            {
                DisconnectFromServer();
                base.OnClosed(e);
            }
        }

        #endregion

        #region TfsBuildLab Proxy Handling

        private IAdministration _adminProxy = null;
        delegate void ShutdownApplicationViaWorkerThread(); 

        private string TfsBuildLabServer
        {
            get
            {
                string tfsBuildLab;

                if (string.IsNullOrEmpty(ConfigurationManager.AppSettings["TFSBuildLab"]))
                {
                    LoginForm login = new LoginForm();
                    if (login.ShowDialog(this) != DialogResult.OK)
                    {
                        this.Close();
                    }
                    tfsBuildLab = login.ServerName;
                }
                else
                {
                    tfsBuildLab = ConfigurationManager.AppSettings["TFSBuildLab"];
                }

                return tfsBuildLab;
            }
        }

        private NetTcpBinding CreateBinding()
        {
            NetTcpBinding binding = new NetTcpBinding(SecurityMode.None);

            binding.ReaderQuotas.MaxArrayLength = int.MaxValue;
            binding.ReaderQuotas.MaxBytesPerRead = int.MaxValue;
            binding.ReaderQuotas.MaxDepth = int.MaxValue;
            binding.ReaderQuotas.MaxStringContentLength = int.MaxValue;
            binding.ReaderQuotas.MaxBytesPerRead = int.MaxValue;

            binding.MaxReceivedMessageSize = int.MaxValue;
            binding.MaxBufferSize = int.MaxValue;

            binding.ReliableSession.Enabled = false;

            return binding;
        }

        private T CreateProxy<T>()
        {
            T proxy = default(T);
            try
            {
                string path = string.Format("net.tcp://{0}/", TfsBuildLabServer);
                ChannelFactory<T> factory = new ChannelFactory<T>(CreateBinding(), path);
                factory.Endpoint.Behaviors.Add(new System.ServiceModel.Description.ClientViaBehavior(new Uri(path)));
                proxy = factory.CreateChannel();
                ICommunicationObject co = (ICommunicationObject)proxy;
                co.Open();
            }
            catch (Exception e)
            {
                MessageBox.Show(string.Format("Failed to connect to TFSBuildLab service on server '{0}'. Reason: {1}", TfsBuildLabServer, e.Message), "Connection failed closing down client!!!", MessageBoxButtons.OK);
                this.Close();
            }

            return proxy;
        }

        private void CloseProxy(ICommunicationObject co)
        {
            if (co != null)
            {
                if (co.State == CommunicationState.Opened)
                {
                    co.Close();
                }
            }
        }

        private bool ConnectToServer()
        {
            try
            {
                string path = string.Format("net.tcp://{0}/", TfsBuildLabServer);
                DuplexChannelFactory<IAdministration> adminFactory = new DuplexChannelFactory<IAdministration>(new InstanceContext(this), CreateBinding(), path);

                CallbackBehaviorAttribute callbackBehaviour = (CallbackBehaviorAttribute)adminFactory.Endpoint.Behaviors[typeof(CallbackBehaviorAttribute)];
                callbackBehaviour.ConcurrencyMode = ConcurrencyMode.Reentrant;

                _adminProxy = adminFactory.CreateChannel();
                _adminProxy.Connect();

                ICommunicationObject co = (ICommunicationObject)_adminProxy;
                co.Faulted += new EventHandler(ServerConnectionBroke);
            }
            catch (Exception e)
            {
                _adminProxy = null;
                MessageBox.Show(string.Format("Failed to connect to TFSBuildLab service on server '{0}'. Reason: {1}", TfsBuildLabServer, e.Message), "Connection failed closing down client!!!", MessageBoxButtons.OK);
                this.Close();
                return false;
            }

            return true;
        }

        private void DisconnectFromServer()
        {
            if (_adminProxy != null)
            {
                _adminProxy.Disconnect();
                CloseProxy((ICommunicationObject)_adminProxy);
            }
        }

        void ServerConnectionBroke(object sender, EventArgs e)
        {
            _adminProxy = null;

            if (MessageBox.Show(string.Format("Connect to TFSBuildLab service on server '{0}' was lost do you wish to try and reconnect?", TfsBuildLabServer), "Server connection unexpectedly faulted!!!", MessageBoxButtons.YesNo) == DialogResult.Yes)
            {
                ConnectToServer();
            }
            else
            {
               this.Invoke(new ShutdownApplicationViaWorkerThread(ShutdownApplication)); 
            }
        }

        void ShutdownApplication()
        {
            this.Close();
        }

        #endregion

        #region IEventNotificationCallback Members

        void IEventNotificationCallback.Notify(SubscriptionService.EventType eventType, string eventXml)
        {

            switch (eventType)
            {
                case SubscriptionService.EventType.CheckinEvent:
                    CheckinEvent checkinEvent = Common.Deserialize<CheckinEvent>(eventXml);
                    ListViewItem lvi = new ListViewItem(checkinEvent.CreationDate);
                    lvi.SubItems.Add(checkinEvent.Owner); 
                    lvi.SubItems.Add(string.Format("Changeset {0} checked-in.", checkinEvent.Number));
                    eventListView.Items.Insert(0, lvi);
                    break;
                case SubscriptionService.EventType.BuildCompletionEvent:
                    BuildCompletionEvent buildCompletionEvent = Common.Deserialize<BuildCompletionEvent>(eventXml);
                    ListViewItem lvi2 = new ListViewItem(buildCompletionEvent.BuildCompleteTime);
                    lvi2.SubItems.Add(buildCompletionEvent.RequestedBy); // Color?
                    lvi2.SubItems.Add(string.Format("Build {0} {1}.", buildCompletionEvent.Id, buildCompletionEvent.CompletionStatus.ToLower()));
                    eventListView.Items.Insert(0, lvi2);
                    break;
                case SubscriptionService.EventType.BuildStatusChangeEvent:
                    BuildStatusChangeEvent buildStatusChangeEvent = Common.Deserialize<BuildStatusChangeEvent>(eventXml);
                    ListViewItem lvi3 = new ListViewItem(buildStatusChangeEvent.ChangedTime);
                    lvi3.SubItems.Add(buildStatusChangeEvent.ChangedBy);
                    lvi3.SubItems.Add(string.Format("Build {0} changed from {1} to {2}.", buildStatusChangeEvent.Id, buildStatusChangeEvent.StatusChange.OldValue, buildStatusChangeEvent.StatusChange.NewValue));
                    eventListView.Items.Insert(0, lvi3);
                    break;
            }

        }

        #endregion

        #region Maximize To Fullscreen Mode

        protected override bool ProcessCmdKey(ref Message msg, Keys keyData)
        {
            if (keyData == Keys.F11)
            {
                ToggleFullScreenMode();
                return true;
            }

            return false;
        }

        private void ToggleFullScreenMode()
        {
            mainTabControl.SuspendLayout();

            if (_fullScreenMode)
            {
                mainTabControl.TabPages[0].Text = "Dashboard";

                //Show tabs
                foreach (TabPage tab in _hiddenTabs)
                {
                    mainTabControl.TabPages.Insert(1, tab);
                }
                _hiddenTabs.Clear();

                mainTabControl.SelectTab(_tabPositionWhenEnteringFullScreenMode);

                MaximizeBox = true;
                MinimizeBox = true;
                TopMost = false;
                FormBorderStyle = System.Windows.Forms.FormBorderStyle.Sizable;
                WindowState = System.Windows.Forms.FormWindowState.Normal;

                _fullScreenMode = false;
            }
            else
            {
                mainTabControl.TabPages[0].Text = "TfsBuildLab Dashboard (Press F11 to exit FullScreen mode)";
                _tabPositionWhenEnteringFullScreenMode = mainTabControl.SelectedIndex;

                //Hide tabs
                for (int index = mainTabControl.TabPages.Count - 1; index > 0; index--)
                {
                    TabPage tab = mainTabControl.TabPages[index];
                    mainTabControl.TabPages.Remove(tab);
                    _hiddenTabs.Add(tab);
                }

                MaximizeBox = false;
                MinimizeBox = false;
                TopMost = true;
                FormBorderStyle = System.Windows.Forms.FormBorderStyle.None;
                WindowState = System.Windows.Forms.FormWindowState.Maximized;

                _fullScreenMode = true;
            }

            mainTabControl.ResumeLayout();
        }
        
        #endregion

        #region Minimize To System Tray Code

        private void MainForm_SizeChanged(object sender, EventArgs e)
        {
            if (this.WindowState == FormWindowState.Minimized)
            {
                this.ShowInTaskbar = false;
                this.trayIcon.Visible = true;
                this.Hide();
            }
        }

        private void TrayIcon_DoubleClick(object sender, EventArgs e)
        {
            this.trayIcon.Visible = false;
            this.WindowState = FormWindowState.Normal;
            this.ShowInTaskbar = true;
            this.SetTopLevel(true);
            this.Show();
            this.Activate();
        }

        private void TrayIcon_ContextMenuItemExitClick(object sender, System.EventArgs e)
        {
            this.trayIcon.Visible = false;
            this.Close();
        }

        #endregion

        #region Dashboard Tab

        private void reportsComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            RenderReport((string)reportsComboBox.SelectedItem);
        }

        private void RenderReport(string reportName)
        {
            this.DashboardReportViewer.ProcessingMode = Microsoft.Reporting.WinForms.ProcessingMode.Remote;
            this.DashboardReportViewer.ServerReport.ReportServerUrl = new Uri(ConfigurationManager.AppSettings["ReportServer"]);
            this.DashboardReportViewer.ServerReport.ReportPath = reportName;
            this.DashboardReportViewer.ShowParameterPrompts = false;
            this.DashboardReportViewer.ShowToolBar = false;
            this.DashboardReportViewer.RefreshReport();
        }

        #endregion

        #region Build Queue Tab

        private void buildQueueTabPage_Enter(object sender, EventArgs e)
        {
            using (new WaitCursor(this))
            {
                RefreshBuildQueue();
            }
        }

        private void refreshBuildQueueGridContextMenuItem_Click(object sender, EventArgs e)
        {
            using (new WaitCursor(this))
            {
                RefreshBuildQueue();
            }
        }

        private void RefreshBuildQueue()
        {
            //TODO figure out why the grid wont loose the first selected row when starting the app (works on second refresh).
            if (!_isInitializing)
            {
                TeamProject teamProject = (TeamProject)BQTeamProject.SelectedValue;
                BuildType buildType = (BuildType)BQBuildType.SelectedValue;

                if (teamProject == null)
                {
                    teamProject = new TeamProject("");
                }

                if (buildType == null)
                {
                    buildType = new BuildType("", "");
                }

                IBuildQueueManagement proxy = null;
                try
                {
                    proxy = CreateProxy<IBuildQueueManagement>();
                    List<QueueEntry> queuedBuilds = proxy.ListQueuedBuilds(teamProject.Name, buildType.Name);
                    SortableBindingList<QueueEntry> queuedBuildsSortable = new SortableBindingList<QueueEntry>(queuedBuilds);
                    BuildQueueGrid.DataSource = queuedBuildsSortable;
                    BuildQueueGrid.Sort(BuildQueueGrid.Columns[_colSortedIndexBuildQueue], _colSortOrderBuildQueue);
                    BuildQueueGrid.Refresh();
                    BuildQueueGrid.ClearSelection();
                }
                catch (FaultException ex)
                {
                    DisplayError(ex);
                }
                finally
                {
                    CloseProxy((ICommunicationObject)proxy);
                }
            }
        }

        private void RemoveBuildFromQueue_Click(object sender, EventArgs e)
        {
            using (new WaitCursor(this))
            {
                if (BuildQueueGrid.SelectedRows.Count == 1)
                {
                    int id = (int)BuildQueueGrid.SelectedRows[0].Cells[0].Value;

                    IBuildQueueManagement proxy = null;
                    try
                    {
                        proxy = CreateProxy<IBuildQueueManagement>();
                        proxy.RemoveBuildFromQueue(id);
                    }
                    catch (FaultException ex)
                    {
                        DisplayError(ex);
                    }
                    finally
                    {
                        CloseProxy((ICommunicationObject)proxy);
                    }

                    RefreshBuildQueue();

                    MoveBuildFirst.Enabled = false;
                    MoveBuildLast.Enabled = false;
                    RemoveBuildFromQueue.Enabled = false;
                }
            }
        }

        private void MoveBuildFirst_Click(object sender, EventArgs e)
        {
            using (new WaitCursor(this))
            {
                if (BuildQueueGrid.SelectedRows.Count == 1)
                {
                    int id = (int)BuildQueueGrid.SelectedRows[0].Cells[0].Value;

                    IBuildQueueManagement proxy = null;
                    try
                    {
                        proxy = CreateProxy<IBuildQueueManagement>();
                        proxy.MoveBuildToBeginingOfQueue(id);
                    }
                    catch (FaultException ex)
                    {
                        DisplayError(ex);
                    }
                    finally
                    {
                        CloseProxy((ICommunicationObject)proxy);
                    }

                    RefreshBuildQueue();

                    MoveBuildFirst.Enabled = false;
                    MoveBuildLast.Enabled = false;
                    RemoveBuildFromQueue.Enabled = false;
                }
            }
        }

        private void MoveBuildLast_Click(object sender, EventArgs e)
        {
            using (new WaitCursor(this))
            {
                if (BuildQueueGrid.SelectedRows.Count == 1)
                {
                    int id = (int)BuildQueueGrid.SelectedRows[0].Cells[0].Value;

                    IBuildQueueManagement proxy = null;
                    try
                    {
                        proxy = CreateProxy<IBuildQueueManagement>();
                        proxy.MoveBuildToEndOfQueue(id);
                    }
                    catch (FaultException ex)
                    {
                        DisplayError(ex);
                    }
                    finally
                    {
                        CloseProxy((ICommunicationObject)proxy);
                    }

                    RefreshBuildQueue();

                    MoveBuildFirst.Enabled = false;
                    MoveBuildLast.Enabled = false;
                    RemoveBuildFromQueue.Enabled = false;
                }
            }
        }

        private void QueueBuild_Click(object sender, EventArgs e)
        {
            using (new WaitCursor(this))
            {
                IBuildTypeAdministration buildTypeProxy = null;
                try
                {
                    buildTypeProxy = CreateProxy<IBuildTypeAdministration>();

                    QueueForm dlg = new QueueForm(buildTypeProxy);
                    if (dlg.ShowDialog() == DialogResult.OK)
                    {
                        using (new WaitCursor(this))
                        {
                            IBuildQueueManagement buildQueueProxy = null;
                            try
                            {
                                buildQueueProxy = CreateProxy<IBuildQueueManagement>();
                                buildQueueProxy.AddBuildToQueue(dlg.SelectedTeamProject, dlg.SelectedBuildType, WindowsIdentity.GetCurrent().Name, dlg.OverriddenBuildParameters);
                            }
                            catch (FaultException ex)
                            {
                                DisplayError(ex);
                            }
                            finally
                            {
                                CloseProxy((ICommunicationObject)buildQueueProxy);
                            }

                            RefreshBuildQueue();
                        }
                    }
                }
                catch (FaultException ex)
                {
                    DisplayError(ex);
                }
                finally
                {
                    CloseProxy((ICommunicationObject)buildTypeProxy);
                }
            }
        }

        private void BuildQueueGrid_CellClick(object sender, DataGridViewCellEventArgs e)
        {
            using (new WaitCursor(this))
            {
                bool state = BuildQueueGrid.SelectedRows.Count == 1;

                MoveBuildFirst.Enabled = state;
                MoveBuildLast.Enabled = state;
                RemoveBuildFromQueue.Enabled = state;
            }
        }

        private void BuildQueueGrid_CellDoubleClick(object sender, DataGridViewCellEventArgs e)
        {
            using (new WaitCursor(this))
            {
                QueueEntry qe = (QueueEntry)BuildQueueGrid.Rows[e.RowIndex].DataBoundItem;

                if (qe.OverriddenBuildParameters.Count > 0)
                {
                    string message = "";
                    
                    foreach (string key in qe.OverriddenBuildParameters.Keys)
                    {
                        message += key + " = " + qe.OverriddenBuildParameters[key] + "\n";
                    }

                    MessageBox.Show(message, "Overriden Build Parameters");
                }
            }
        }

        private void BuildQueueGrid_ColumnHeaderMouseClick(object sender, DataGridViewCellMouseEventArgs e)
        {
            if (BuildQueueGrid.SortedColumn != null)
            {
                _colSortedIndexBuildQueue = BuildQueueGrid.SortedColumn.Index;
                if (BuildQueueGrid.SortOrder == SortOrder.Descending)
                {
                    _colSortOrderBuildQueue = ListSortDirection.Descending;
                }
                else
                {
                    _colSortOrderBuildQueue = ListSortDirection.Ascending;
                }
            }
        }

        private void BQTeamProject_SelectedIndexChanged(object sender, EventArgs e)
        {
            using (new WaitCursor(this))
            {
                TeamProject project = (TeamProject)BQTeamProject.SelectedValue;
                if (project != null)
                {
                    PopulateBuildTypes(project, BQBuildType);
                }
            }
        }

        private void BQBuildType_SelectedIndexChanged(object sender, EventArgs e)
        {
            using (new WaitCursor(this))
            {
                RefreshBuildQueue();
            }
        }

        #endregion

        #region Build Log Tab

        private void buildLogTabPage_Enter(object sender, EventArgs e)
        {
            using (new WaitCursor(this))
            {
                RefreshBuildLog();
            }
        }

        private void refreshBuildLogGridContextMenuItem_Click(object sender, EventArgs e)
        {
            using (new WaitCursor(this))
            {
                RefreshBuildLog();
            }
        }

        private void RefreshBuildLog()
        {
            //TODO figure out why the grid wont loose the first selected row when starting the app (works on second refresh).
            if (!_isInitializing)
            {
                TeamProject teamProject = (TeamProject)BLTeamProject.SelectedValue;
                BuildType buildType = (BuildType)BLBuildType.SelectedValue;
                string buildStatus = BLBuildResult.Text;
                
                if (teamProject == null)
                {
                    teamProject = new TeamProject("");
                }

                if (buildType == null)
                {
                    buildType = new BuildType("", "");
                }

                IBuildRepositoryManagement proxy = null;
                try
                {
                    proxy = CreateProxy<IBuildRepositoryManagement>();
                    List<TeamBuild> builds = proxy.ListBuilds(teamProject.Name, buildType.Name, buildStatus);
                    SortableBindingList<TeamBuild> buildsSortable = new SortableBindingList<TeamBuild>(builds);
                    BuildLogGrid.DataSource = buildsSortable;
                    BuildLogGrid.Sort(BuildLogGrid.Columns[_colSortedIndexBuildLog], _colSortOrderBuildLog);
                    BuildLogGrid.Refresh();
                    BuildLogGrid.ClearSelection();
                }
                catch (FaultException ex)
                {
                    DisplayError(ex);
                }
                finally
                {
                    CloseProxy((ICommunicationObject)proxy);
                }
            }
        }

        private void DeleteBuild_Click(object sender, EventArgs e)
        {
            using (new WaitCursor(this))
            {
                IBuildRepositoryManagement proxy = null;
                try
                {
                    proxy = CreateProxy<IBuildRepositoryManagement>();

                    List<string> buildUris = new List<string>();
                    foreach (DataGridViewRow row in BuildLogGrid.SelectedRows)
                    {
                        buildUris.Add((string)row.Cells[0].Value);
                    }

                    if (buildUris.Count > 1)
                    {
                        ProgressForm dlg = new ProgressForm("Deleting Builds...", buildUris, proxy.DeleteBuild);
                        dlg.ShowDialog();
                    }
                    else
                    {
                        proxy.DeleteBuild(buildUris[0]);
                    }
                }
                catch (FaultException ex)
                {
                    DisplayError(ex);
                }
                finally
                {
                    CloseProxy((ICommunicationObject)proxy);
                }

                RefreshBuildLog();

                StopBuild.Enabled = false;
                DeleteBuild.Enabled = false;
            }
        }

        private void StopBuild_Click(object sender, EventArgs e)
        {
            using (new WaitCursor(this))
            {
                IBuildRepositoryManagement proxy = null;
                try
                {
                    proxy = CreateProxy<IBuildRepositoryManagement>();

                    List<string> buildUris = new List<string>();
                    foreach (DataGridViewRow row in BuildLogGrid.SelectedRows)
                    {
                        buildUris.Add((string)row.Cells[0].Value);
                    }

                    if (buildUris.Count > 1)
                    {
                        ProgressForm dlg = new ProgressForm("Stopping Builds...", buildUris, proxy.StopBuild);
                        dlg.ShowDialog();
                    }
                    else
                    {
                        proxy.StopBuild(buildUris[0]);
                    }
                }
                catch (FaultException ex)
                {
                    DisplayError(ex);
                }
                finally
                {
                    CloseProxy((ICommunicationObject)proxy);
                }
                
                RefreshBuildLog();

                StopBuild.Enabled = false;
                DeleteBuild.Enabled = false;
            }
        }

        private void BuildLogGrid_CellClick(object sender, DataGridViewCellEventArgs e)
        {
            using (new WaitCursor(this))
            {
                bool state = BuildLogGrid.SelectedRows.Count > 0;

                StopBuild.Enabled = state;
                DeleteBuild.Enabled = state;
            }
        }

        private void BuildLogGrid_ColumnHeaderMouseClick(object sender, DataGridViewCellMouseEventArgs e)
        {
            if (BuildLogGrid.SortedColumn != null)
            {
                _colSortedIndexBuildLog = BuildLogGrid.SortedColumn.Index;
                if (BuildLogGrid.SortOrder == SortOrder.Descending)
                {
                    _colSortOrderBuildLog = ListSortDirection.Descending;
                }
                else
                {
                    _colSortOrderBuildLog = ListSortDirection.Ascending;
                }
            }
        }

        private void BLTeamProject_SelectedIndexChanged(object sender, EventArgs e)
        {
            using (new WaitCursor(this))
            {
                TeamProject project = (TeamProject)BLTeamProject.SelectedValue;
                if (project != null)
                {
                    PopulateBuildTypes(project, BLBuildType);
                }
            }
        }

        private void BLBuildType_SelectedIndexChanged(object sender, EventArgs e)
        {
            using (new WaitCursor(this))
            {
                RefreshBuildLog();
            }
        }

        private void BLBuildResult_SelectedIndexChanged(object sender, EventArgs e)
        {
            using (new WaitCursor(this))
            {
                RefreshBuildLog();
            }
        }

        #endregion

        #region Build Events Tab

        private void historyTabPage_Enter(object sender, EventArgs e)
        {
            using (new WaitCursor(this))
            {
                RefreshBuildEvents();
            }
        }

        private void refreshBuildEventsGridContextMenuItem_Click(object sender, EventArgs e)
        {
            using (new WaitCursor(this))
            {
                RefreshBuildEvents();
            }
        }

        private void RefreshBuildEvents()
        {
            //TODO figure out why the grid wont loose the first selected row when starting the app (works on second refresh).
            if (!_isInitializing)
            {
                TeamProject teamProject = (TeamProject)BHTeamProject.SelectedValue;
                BuildType buildType = (BuildType)BHBuildType.SelectedValue;
                string eventType = BHEventType.Text;
                DateTime startDTM = BHStartDTM.Value;
                DateTime endDTM = BHEndDTM.Value;

                if (teamProject == null)
                {
                    teamProject = new TeamProject("");
                }

                if (buildType == null)
                {
                    buildType = new BuildType("", "");
                }

                IStatistics proxy = null;
                try
                {
                    proxy = CreateProxy<IStatistics>();
                    List<LogEntry> events = proxy.ListEvents(teamProject.Name, buildType.Name, eventType, startDTM, endDTM);
                    SortableBindingList<LogEntry> eventsSortable = new SortableBindingList<LogEntry>(events);
                    BuildEventLog.DataSource = eventsSortable;
                    BuildEventLog.Sort(BuildEventLog.Columns[_colSortedIndexBuildEventLog], _colSortOrderBuildEventLog);
                    BuildEventLog.Refresh();
                    BuildEventLog.ClearSelection();
                }
                catch (FaultException ex)
                {
                    DisplayError(ex);
                }
                finally
                {
                    CloseProxy((ICommunicationObject)proxy);
                }
            }
        }

        private void BuildEventLog_ColumnHeaderMouseClick(object sender, DataGridViewCellMouseEventArgs e)
        {
            if (BuildEventLog.SortedColumn != null)
            {
                _colSortedIndexBuildEventLog = BuildEventLog.SortedColumn.Index;
                if (BuildEventLog.SortOrder == SortOrder.Descending)
                {
                    _colSortOrderBuildEventLog = ListSortDirection.Descending;
                }
                else
                {
                    _colSortOrderBuildEventLog = ListSortDirection.Ascending;
                }
            }
        }

        private void BHTeamProject_SelectedIndexChanged(object sender, EventArgs e)
        {
            using (new WaitCursor(this))
            {
                TeamProject project = (TeamProject)BHTeamProject.SelectedValue;
                if (project != null)
                {
                    PopulateBuildTypes(project, BHBuildType);
                }
            }
        }

        private void BHBuildType_SelectedIndexChanged(object sender, EventArgs e)
        {
            using (new WaitCursor(this))
            {
                RefreshBuildEvents();
            }
        }

        private void BHEventType_SelectedIndexChanged(object sender, EventArgs e)
        {
            using (new WaitCursor(this))
            {
                RefreshBuildEvents();
            }
        }

        #endregion

        #region Configuration Tab

        void contextMenuBuildTypeInfo_Popup(object sender, EventArgs e)
        {
            string teamProject = "";
            string buildType = "";

            if (BuildTypeTreeView.SelectedNode != null)
            {
                buildType = BuildTypeTreeView.SelectedNode.Text;
            }

            if (BuildTypeTreeView.SelectedNode.Parent != null)
            {
                teamProject = BuildTypeTreeView.SelectedNode.Parent.Text;
            }

            if (!string.IsNullOrEmpty(teamProject) && !string.IsNullOrEmpty(buildType))
            {
                menuItemCopyBuildTypeInfo.Enabled = true;
            }
            else
            {
                menuItemCopyBuildTypeInfo.Enabled = false;
            }
        }

        private void copyBuildTypeInfoContextMenuItem_Click(object sender, EventArgs e)
        {
            string sourceTeamProject = BuildTypeTreeView.SelectedNode.Parent.Text;
            string sourceBuildType = BuildTypeTreeView.SelectedNode.Text;

            IBuildTypeAdministration proxy = null;
            try
            {
                proxy = CreateProxy<IBuildTypeAdministration>();

                SelectorForm dlg = new SelectorForm("Pleas select a build type to copy", proxy);
                if (dlg.ShowDialog() == DialogResult.OK)
                {
                    using (new WaitCursor(this))
                    {
                        proxy.CopyBuildType(sourceTeamProject, sourceBuildType, dlg.SelectedTeamProject, dlg.SelectedBuildType);
                    }
                }
            }
            catch (FaultException ex)
            {
                DisplayError(ex);
            }
            finally
            {
                CloseProxy((ICommunicationObject)proxy);
            }
        }

        private void refreshBuildTypeInfoContextMenuItem_Click(object sender, EventArgs e)
        {
            using (new WaitCursor(this))
            {
                List<TeamProject> teamProjects = null;
                IBuildTypeAdministration proxy = null;
                
                try
                {
                    proxy = CreateProxy<IBuildTypeAdministration>();
                    teamProjects = proxy.ListTeamProjects();
                }
                catch (FaultException ex)
                {
                    DisplayError(ex);
                }
                finally
                {
                    CloseProxy((ICommunicationObject)proxy);
                }

                PopulateTeamProjects(teamProjects, BuildTypeTreeView);

                RetentionPolicyGridView.SuspendLayout();
                RetentionPolicyGridView.DataSource = null;
                RetentionPolicyGridView.ClearSelection();
                RetentionPolicyGridView.ResumeLayout();

                TriggersGridView.SuspendLayout();
                TriggersGridView.DataSource = null;
                TriggersGridView.ClearSelection();
                TriggersGridView.ResumeLayout();
            }
        }

        private void RefreshBuildTypeInfo()
        {
            using (new WaitCursor(this))
            {
                string teamProject = BuildTypeTreeView.SelectedNode.Parent.Text;
                string buildType = BuildTypeTreeView.SelectedNode.Text;

                BuildType bt = null;
                IBuildTypeAdministration proxy = null;

                try
                {
                    proxy = CreateProxy<IBuildTypeAdministration>();
                    bt = proxy.LoadBuildType(teamProject, buildType);
                }
                catch (FaultException ex)
                {
                    DisplayError(ex);
                }
                finally
                {
                    CloseProxy((ICommunicationObject)proxy);
                }

                List<RetentionPolicy> policies = new List<RetentionPolicy>();
                foreach (RetentionPolicy p in bt.RetentionPolicies.Values)
                {
                    policies.Add(p);
                }

                RetentionPolicyGridView.SuspendLayout();
                RetentionPolicyGridView.DataSource = null;
                RetentionPolicyGridView.DataSource = policies;
                RetentionPolicyGridView.ClearSelection();
                RetentionPolicyGridView.ResumeLayout();

                List<Trigger> triggers = new List<Trigger>();
                foreach (Trigger t in bt.Triggers.Values)
                {
                    triggers.Add(t);
                }

                TriggersGridView.SuspendLayout();
                TriggersGridView.DataSource = null;
                TriggersGridView.DataSource = triggers;
                TriggersGridView.ClearSelection();
                TriggersGridView.ResumeLayout();
            }
        }

        private void NewRetentionPolicy_Click(object sender, EventArgs e)
        {
            using (new WaitCursor(this))
            {
                string teamProject = BuildTypeTreeView.SelectedNode.Parent.Text;
                string buildType = BuildTypeTreeView.SelectedNode.Text;

                IRetentionPolicyAdministration proxy = null;
                try
                {
                    proxy = CreateProxy<IRetentionPolicyAdministration>();

                    RetentionPolicyForm dlg = new RetentionPolicyForm(proxy, teamProject, buildType);
                    if (dlg.ShowDialog() == DialogResult.OK)
                    {
                        RefreshBuildTypeInfo();
                    }
                }
                catch (FaultException ex)
                {
                    DisplayError(ex);
                }
                finally
                {
                    CloseProxy((ICommunicationObject)proxy);
                }
            }
        }

        private void NewTrigger_Click(object sender, EventArgs e)
        {
            using (new WaitCursor(this))
            {
                string teamProject = BuildTypeTreeView.SelectedNode.Parent.Text;
                string buildType = BuildTypeTreeView.SelectedNode.Text;

                IBuildTypeAdministration buildTypeProxy = null;
                ITriggerAdministration triggerProxy = null;
                try
                {
                    buildTypeProxy = CreateProxy<IBuildTypeAdministration>();
                    triggerProxy = CreateProxy<ITriggerAdministration>();

                    TriggerForm dlg = new TriggerForm(triggerProxy, buildTypeProxy, teamProject, buildType);
                    if (dlg.ShowDialog() == DialogResult.OK)
                    {
                        RefreshBuildTypeInfo();
                    }
                }
                catch (FaultException ex)
                {
                    DisplayError(ex);
                }
                finally
                {
                    CloseProxy((ICommunicationObject)buildTypeProxy);
                    CloseProxy((ICommunicationObject)triggerProxy);
                }
            }
        }

        private void DelRetentionPolicy_Click(object sender, EventArgs e)
        {
            using (new WaitCursor(this))
            {
                int id = (int)RetentionPolicyGridView.SelectedRows[0].Cells[0].Value;
                IRetentionPolicyAdministration proxy = null;
                try
                {
                    proxy = CreateProxy<IRetentionPolicyAdministration>();
                    proxy.DeleteRetentionPolicy(id);
                }
                catch (FaultException ex)
                {
                    DisplayError(ex);
                }
                finally
                {
                    CloseProxy((ICommunicationObject)proxy);
                }

                RefreshBuildTypeInfo();
                DelRetentionPolicy.Enabled = false;
            }
        }

        private void DelTrigger_Click(object sender, EventArgs e)
        {
            using (new WaitCursor(this))
            {
                int id = (int)TriggersGridView.SelectedRows[0].Cells[0].Value;
                ITriggerAdministration proxy = null;
                try
                {
                    proxy = CreateProxy<ITriggerAdministration>();
                    proxy.DeleteTrigger(id);
                }
                catch (FaultException ex)
                {
                    DisplayError(ex);
                }
                finally
                {
                    CloseProxy((ICommunicationObject)proxy);
                }

                RefreshBuildTypeInfo();
                DelTrigger.Enabled = false;
            }
        }

        private void RetentionPolicyGridView_CellClick(object sender, DataGridViewCellEventArgs e)
        {
            using (new WaitCursor(this))
            {
                DelRetentionPolicy.Enabled = true;
            }
        }

        private void RetentionPolicyGridView_CellDoubleClick(object sender, DataGridViewCellEventArgs e)
        {
            using (new WaitCursor(this))
            {
                RetentionPolicy rp = RetentionPolicyGridView.Rows[e.RowIndex].DataBoundItem as RetentionPolicy;
                IRetentionPolicyAdministration proxy = null;
                try
                {
                    proxy = CreateProxy<IRetentionPolicyAdministration>();

                    RetentionPolicyForm dlg = new RetentionPolicyForm(proxy, rp);
                    if (dlg.ShowDialog() == DialogResult.OK)
                    {
                        RefreshBuildTypeInfo();
                    }
                }
                catch (FaultException ex)
                {
                    DisplayError(ex);
                }
                finally
                {
                    CloseProxy((ICommunicationObject)proxy);
                }
            }
        }

        private void TriggersGridView_CellClick(object sender, DataGridViewCellEventArgs e)
        {
            using (new WaitCursor(this))
            {
                DelTrigger.Enabled = true;
            }
        }

        private void TriggersGridView_CellDoubleClick(object sender, DataGridViewCellEventArgs e)
        {
            using (new WaitCursor(this))
            {
                Trigger t = TriggersGridView.Rows[e.RowIndex].DataBoundItem as Trigger;

                string teamProject = BuildTypeTreeView.SelectedNode.Parent.Text;
                string buildType = BuildTypeTreeView.SelectedNode.Text;

                IBuildTypeAdministration buildTypeProxy = null;
                ITriggerAdministration triggerProxy = null;
                try
                {
                    buildTypeProxy = CreateProxy<IBuildTypeAdministration>();
                    triggerProxy = CreateProxy<ITriggerAdministration>();

                    TriggerForm dlg = new TriggerForm(triggerProxy, buildTypeProxy, t, teamProject, buildType);
                    if (dlg.ShowDialog() == DialogResult.OK)
                    {
                        RefreshBuildTypeInfo();
                    }
                }
                catch (FaultException ex)
                {
                    DisplayError(ex);
                }
                finally
                {
                    CloseProxy((ICommunicationObject)buildTypeProxy);
                    CloseProxy((ICommunicationObject)triggerProxy);
                }
            }
        }

        private void BuildTypeTreeView_AfterSelect(object sender, TreeViewEventArgs e)
        {
            using (new WaitCursor(this))
            {
                RetentionPolicyGridView.DataSource = null;
                TriggersGridView.DataSource = null;

                if (e.Node.Level == 1)
                {
                    RefreshBuildTypeInfo();

                    NewRetentionPolicy.Enabled = true;
                    NewTrigger.Enabled = true;
                }
                else
                {
                    NewRetentionPolicy.Enabled = false;
                    NewTrigger.Enabled = false;
                }
            }
        }

        #endregion

    }

    public class WaitCursor : IDisposable
    {
        private Form _form;

        public WaitCursor(Form form)
        {
            _form = form;
            _form.Cursor = Cursors.WaitCursor;
        }

        public void Dispose()
        {
            if (_form != null)
                _form.Cursor = Cursors.Default;
        }
    }

    public class SortableBindingList<T> : BindingList<T>
    {
        private PropertyDescriptor propertyDescriptor;
        private ListSortDirection listSortDirection;
        private bool isSorted;

        public SortableBindingList()
        {
        }

        public SortableBindingList(List<T> source)
        {
            foreach (T item in source)
                this.Add(item);
        }

        protected override bool SupportsSortingCore
        {
            get { return true; }
        }

        protected override bool IsSortedCore
        {
            get { return this.isSorted; }
        }

        protected override PropertyDescriptor SortPropertyCore
        {
            get { return this.propertyDescriptor; }
        }

        protected override ListSortDirection SortDirectionCore
        {
            get { return this.listSortDirection; }
        }

        protected override void ApplySortCore(PropertyDescriptor prop, ListSortDirection direction)
        {
            List<T> itemsList = this.Items as List<T>;
            itemsList.Sort(delegate(T t1, T t2)
            {
                this.propertyDescriptor = prop;
                this.listSortDirection = direction;
                this.isSorted = true;

                int reverse = direction == ListSortDirection.Ascending ? 1 : -1;

                PropertyInfo propertyInfo = typeof(T).GetProperty(prop.Name);
                object value1 = propertyInfo.GetValue(t1, null);
                object value2 = propertyInfo.GetValue(t2, null);

                IComparable comparable = value1 as IComparable;
                if (comparable != null)
                {
                    return reverse * comparable.CompareTo(value2);
                }
                else
                {
                    comparable = value2 as IComparable;
                    if (comparable != null)
                    {
                        return -1 * reverse * comparable.CompareTo(value1);
                    }
                    else
                    {
                        return 0;
                    }
                }
            });

            this.OnListChanged(new ListChangedEventArgs(ListChangedType.Reset, -1));
        }

        protected override void RemoveSortCore()
        {
            this.isSorted = false;
            this.propertyDescriptor = base.SortPropertyCore;
            this.listSortDirection = base.SortDirectionCore;
        }
    }
}
