using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Threading;
using System.Windows.Forms;
using Callista.BuildLab.Core;
using Callista.BuildLab.Core.Model;

namespace Callista.BuildLab.AdminClient
{
    public partial class ProgressForm : Form
    {
        public delegate string ProcessBuildDelegate(string buildUri);

        //TODO test this with anonymous delegates
        private delegate void IncrementDelegate();
        private delegate void ShutdownDelegate();

        private List<string> _buildUris = null;
        private bool _isAborting = false;
        private Thread _workerThread = null;
        private ProcessBuildDelegate _processBuildFunction = null;

        public ProgressForm(string caption, List<string> buildUris, ProcessBuildDelegate processBuildFunction)
        {
            InitializeComponent();
            
            _buildUris = buildUris;
            _processBuildFunction = processBuildFunction;

            this.BuildsProgressBar.Minimum = 1;
            this.BuildsProgressBar.Maximum = _buildUris.Count;
            this.BuildsProgressBar.Step = 1;
            this.BuildsProgressBar.Value = 1;

            this.Text = caption;
            this.Message.Text = "Processing " + _buildUris[0] + "   [1/"+ _buildUris.Count + "]";
        }

        protected override void OnShown(EventArgs e)
        {
            base.OnShown(e);

            _workerThread = new Thread(this.ProcessBuilds);
            _workerThread .Start();
        }

        private void ProcessBuilds()
        {
            foreach(string uri in _buildUris)
            {
                if(!_isAborting)
                {
                    _processBuildFunction.Invoke(uri);
                    this.Invoke(new IncrementDelegate(IncrementCounter));
                }
                else
                {
                    break;
                }
            }

            this.Invoke(new ShutdownDelegate(CloseDialog));
        }

        private void CloseDialog()
        {
            this.Close();
        }

        private void IncrementCounter()
        {
            this.BuildsProgressBar.Increment(1);
            this.Message.Text = "Processing " + _buildUris[this.BuildsProgressBar.Value-1] + "   [" + this.BuildsProgressBar.Value + "/" + _buildUris.Count + "]";
        }

        private void Abort_Click(object sender, EventArgs e)
        {
            _isAborting = true;
        }

        private void ProgressForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            _isAborting = true;
        }
    }
}