using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Callista.BuildLab.Core;
using Callista.BuildLab.Core.Model;

namespace Callista.BuildLab.AdminClient
{
    public partial class QueueForm : Form
    {
        private IBuildTypeAdministration _buildTypeAdminProxy = null;

        private string _teamProject;
        public string SelectedTeamProject
        {
            get { return _teamProject; }
        }

        private string _buildType;
        public string SelectedBuildType
        {
            get { return _buildType; }
        }

        private Dictionary<string,string> _buildParameters = new Dictionary<string,string>();
        public Dictionary<string, string> OverriddenBuildParameters
        {
            get { return _buildParameters; }
        }

        public QueueForm(IBuildTypeAdministration buildTypeAdminProxy)
        {
            _buildTypeAdminProxy = buildTypeAdminProxy;
            InitializeComponent();
        }

        private void QueueForm_Load(object sender, EventArgs e)
        {
            TeamProjects.DataSource = _buildTypeAdminProxy.ListTeamProjects();
            TeamProjects.DisplayMember = "Name";
            
            Parameters.Visible = false;

            UpdateLayout();
        }

        private void OK_Click(object sender, EventArgs e)
        {
            TeamProject teamProject = (TeamProject)TeamProjects.SelectedValue;
            BuildType buildType = (BuildType)BuildTypes.SelectedValue;

            if (buildType != null && teamProject != null)
            {
                _teamProject = teamProject.Name;
                _buildType = buildType.Name;

                foreach (DataGridViewRow row in Parameters.Rows)
                {
                    string value = (string)row.Cells[2].Value;
                    if (!String.IsNullOrEmpty(value))
                    {
                        _buildParameters.Add((string)row.Cells[0].Value, (string)row.Cells[2].Value);
                    }
                }
            }

            this.Close();
        }

        private void Cancel_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void TeamProjects_SelectedIndexChanged(object sender, EventArgs e)
        {
            TeamProject teamProject = (TeamProject)TeamProjects.SelectedValue;
            if (teamProject != null)
            {
                BuildTypes.DataSource = _buildTypeAdminProxy.ListBuildTypes(teamProject.Name);
                BuildTypes.DisplayMember = "Name";
            }
        }

        private void BuildTypes_SelectedIndexChanged(object sender, EventArgs e)
        {
            BuildType buildType = (BuildType)BuildTypes.SelectedValue;
            if (buildType != null)
            {
                if (Parameters.DataSource == null)
                {
                    DataGridViewTextBoxColumn col1 = new DataGridViewTextBoxColumn();
                    col1.Name = "Name";
                    col1.DataPropertyName = "Name";
                    Parameters.Columns.Add(col1);

                    DataGridViewTextBoxColumn col2 = new DataGridViewTextBoxColumn();
                    col2.Name = "Original Value";
                    col2.DataPropertyName = "Value";
                    Parameters.Columns.Add(col2);

                    DataGridViewTextBoxColumn col3 = new DataGridViewTextBoxColumn();
                    col3.Name = "New Value";
                    col3.AutoSizeMode = DataGridViewAutoSizeColumnMode.Fill;
                    Parameters.Columns.Add(col3);

                    Parameters.AutoGenerateColumns = false;
                    Parameters.ClearSelection();
                    Parameters.ScrollBars = ScrollBars.Both;
                }
                Parameters.DataSource = buildType.Parameters;
            }
      }

        private void ToggleParameters_Click(object sender, EventArgs e)
        {
            Parameters.Visible = !Parameters.Visible;
            UpdateLayout();
        }

        private void UpdateLayout()
        {
            if (Parameters.Visible)
            {
                OK.Top = 10 + Parameters.Top + Parameters.Height;
                Cancel.Top = 10 + Parameters.Top + Parameters.Height;
                ToggleParameters.Top = 10 + Parameters.Top + Parameters.Height;
                this.Height = 40 + ToggleParameters.Top + ToggleParameters.Height;
                ToggleParameters.Text = "<< Simple Mode";
            }
            else
            {
                OK.Top = 10 + BuildTypes.Top + BuildTypes.Height;
                Cancel.Top = 10 + BuildTypes.Top + BuildTypes.Height;
                ToggleParameters.Top = 10 + BuildTypes.Top + BuildTypes.Height;
                this.Height = 40 + ToggleParameters.Top + ToggleParameters.Height;
                ToggleParameters.Text = "Advanced Mode >>";
            }
        }
    }
}