using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Callista.BuildLab.Core;
using Callista.BuildLab.Core.Model;

namespace Callista.BuildLab.AdminClient
{
    public partial class RetentionPolicyForm : Form
    {
        private IRetentionPolicyAdministration _adminProxy = null;
        
        private string _teamProject = "";
        private string _buildType = "";
        private RetentionPolicy _policy = null;

        public RetentionPolicyForm(IRetentionPolicyAdministration proxy, RetentionPolicy policy)
        {
            _adminProxy = proxy;
            _policy = policy;

            InitializeComponent();

            this.Text = "Update retention policy";
        }

        public RetentionPolicyForm(IRetentionPolicyAdministration proxy, string teamProject, string buildType)
        {
            _adminProxy = proxy;
            _teamProject = teamProject;
            _buildType = buildType;
            
            InitializeComponent();

            this.Text = "Add retention policy";
        }

        private void AddRetentionPolicy_Load(object sender, EventArgs e)
        {
            using (new WaitCursor(this))
            {
                TeamBuildStatus.Items.Add(BuildStatus.Any);
                TeamBuildStatus.Items.Add(BuildStatus.BuildFailed);
                TeamBuildStatus.Items.Add(BuildStatus.BuildStopped);
                TeamBuildStatus.Items.Add(BuildStatus.BuildSucceeded);
                TeamBuildStatus.SelectedIndex = 0;

                TeamBuildQuality.Items.Add(BuildQuality.Any);
                TeamBuildQuality.Items.Add(BuildQuality.Unexamined);
                TeamBuildQuality.Items.Add(BuildQuality.Rejected);
                TeamBuildQuality.Items.Add(BuildQuality.UnderInvestigation);
                TeamBuildQuality.Items.Add(BuildQuality.ReadyforInitialTest);
                TeamBuildQuality.Items.Add(BuildQuality.InitialTestPassed);
                TeamBuildQuality.Items.Add(BuildQuality.LabTestPassed);
                TeamBuildQuality.Items.Add(BuildQuality.UserAcceptanceTestsPassed);
                TeamBuildQuality.Items.Add(BuildQuality.ReadyforDeployment);
                TeamBuildQuality.Items.Add(BuildQuality.Released);
                TeamBuildQuality.SelectedIndex = 0;

                RetentionType.Items.Add(RetentionPolicyType.Builds);
                RetentionType.Items.Add(RetentionPolicyType.Days);
                RetentionType.SelectedIndex = 0;

                if (_policy != null)
                {
                    TeamBuildStatus.Text = _policy.BuildStatus.ToString();
                    TeamBuildQuality.Text = _policy.BuildQuality.ToString();
                    RetentionType.Text = _policy.Type.ToString();
                    RetentionQuantity.Text = _policy.RetentionQuantity.ToString();
                }
            }
        }

        private void OK_Click(object sender, EventArgs e)
        {
            using (new WaitCursor(this))
            {
                if (_policy != null)
                {
                    _policy.BuildStatus = (BuildStatus)Enum.Parse(typeof(BuildStatus), TeamBuildStatus.Text);
                    _policy.BuildQuality = (BuildQuality)Enum.Parse(typeof(BuildQuality), TeamBuildQuality.Text);
                    _policy.Type = (RetentionPolicyType)Enum.Parse(typeof(RetentionPolicyType), RetentionType.Text);
                    _policy.RetentionQuantity = Int32.Parse(RetentionQuantity.Text);
                    _adminProxy.UpdateRetentionPolicy(_policy);
                }
                else
                {
                    _adminProxy.AddRetentionPolicyToBuildType(_teamProject, _buildType, (BuildStatus)Enum.Parse(typeof(BuildStatus), TeamBuildStatus.Text), (BuildQuality)Enum.Parse(typeof(BuildQuality), TeamBuildQuality.Text), (RetentionPolicyType)Enum.Parse(typeof(RetentionPolicyType), RetentionType.Text), Int32.Parse(RetentionQuantity.Text));
                }
                this.Close();
            }
        }

        private void Cancel_Click(object sender, EventArgs e)
        {
            using (new WaitCursor(this))
            {
                this.Close();
            }
        }

        private void RetentionQuantity_TextChanged(object sender, EventArgs e)
        {
            using (new WaitCursor(this))
            {
                OK.Enabled = RetentionQuantity.Text != String.Empty;
            }
        }
    }
}