using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Callista.BuildLab.Core;
using Callista.BuildLab.Core.Model;

namespace Callista.BuildLab.AdminClient
{
    public partial class TriggerForm : Form
    {
        private IBuildTypeAdministration _buildTypeProxy = null;
        private ITriggerAdministration _triggerProxy = null;

        private string _teamProject = "";
        private string _buildType = "";
        private Trigger _trigger = null;

        public TriggerForm(ITriggerAdministration triggerProxy, IBuildTypeAdministration buildTypeProxy, Trigger trigger, string teamProject, string buildType)
        {
            _triggerProxy = triggerProxy;
            _buildTypeProxy = buildTypeProxy;
            
            _trigger = trigger;
            _teamProject = teamProject;
            _buildType = buildType;

            InitializeComponent();

            this.Text = "Update trigger";
        }

        public TriggerForm(ITriggerAdministration triggerProxy, IBuildTypeAdministration buildTypeProxy, string teamProject, string buildType)
        {
            _triggerProxy = triggerProxy;
            _buildTypeProxy = buildTypeProxy;

            _teamProject = teamProject;
            _buildType = buildType;

            InitializeComponent();

            this.Text = "Add trigger";
        }

        private void AddTrigger_Load(object sender, EventArgs e)
        {
            using (new WaitCursor(this))
            {
                Type.Items.Add(TriggerType.CheckinInclude);
                Type.Items.Add(TriggerType.CheckinExclude);
                Type.Items.Add(TriggerType.Schedule);
                
                Type.SelectedIndex = 0;
                groupTriggerParameters.Visible = false;
                groupBuildServers.Visible = false;

                DataGridViewTextBoxColumn tpCol1 = new DataGridViewTextBoxColumn();
                tpCol1.Name = "Name";
                tpCol1.DataPropertyName = "Name";
                TriggerParameters.Columns.Add(tpCol1);

                DataGridViewTextBoxColumn tpCol2 = new DataGridViewTextBoxColumn();
                tpCol2.Name = "Original Value";
                tpCol2.DataPropertyName = "Value";
                TriggerParameters.Columns.Add(tpCol2);

                DataGridViewTextBoxColumn tpCol3 = new DataGridViewTextBoxColumn();
                tpCol3.Name = "New Value";
                tpCol3.AutoSizeMode = DataGridViewAutoSizeColumnMode.Fill;
                TriggerParameters.Columns.Add(tpCol3);

                TriggerParameters.AutoGenerateColumns = false;
                TriggerParameters.ClearSelection();
                TriggerParameters.ScrollBars = ScrollBars.Both;
                TriggerParameters.DataSource = _buildTypeProxy.LoadBuildType(_teamProject,_buildType).Parameters;

                DataGridViewTextBoxColumn bsCol1 = new DataGridViewTextBoxColumn();
                bsCol1.Name = "Server";
                bsCol1.DataPropertyName = "Server";
                BuildServers.Columns.Add(bsCol1);

                BuildServers.AutoGenerateColumns = false;
                BuildServers.ClearSelection();
                BuildServers.ScrollBars = ScrollBars.Both;

                if (_trigger != null)
                {
                    Type.Enabled = false;
                    Type.Text = _trigger.Type.ToString();
                    Data.Text = _trigger.Data;
                    PopulateOverriddenParameters();
                    PopulateBuildServers();
                }

                UpdateLayout();
            }
        }

        private void OK_Click(object sender, EventArgs e)
        {
            using (new WaitCursor(this))
            {
                Dictionary<string, string> parameters = ExtractOverriddenParameters();
                List<string> servers = ExtractBuildServers();

                if (_trigger != null)
                {
                    _trigger.Type = (TriggerType)Enum.Parse(typeof(TriggerType), Type.Text);
                    _trigger.Data = Data.Text;
                    _trigger.Servers = servers;
                    _trigger.Parameters = parameters;
                    _triggerProxy.UpdateTrigger(_trigger);
                }
                else
                {
                    _trigger = new Trigger((TriggerType)Enum.Parse(typeof(TriggerType), Type.Text), Data.Text, servers, parameters);
                    _triggerProxy.AddTriggerToBuildType(_teamProject, _buildType, _trigger);
                }
                this.Close();
            }
        }

        private void Cancel_Click(object sender, EventArgs e)
        {
            using (new WaitCursor(this))
            {
                this.Close();
            }
        }

        private void TriggerData_TextChanged(object sender, EventArgs e)
        {
            using (new WaitCursor(this))
            {
                OK.Enabled = Data.Text != String.Empty;
            }
        }

        private void Type_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (Type.Text == TriggerType.CheckinInclude.ToString())
            {
                ToggleMode.Visible = true;
                ScheaduleOverview.Visible = false;
            }
            else
            {
                if (Type.Text == TriggerType.Schedule.ToString())
                {
                    ScheaduleOverview.Visible = true;
                }
                else
                {
                    ScheaduleOverview.Visible = false;
                }
                groupTriggerParameters.Visible = false;
                groupBuildServers.Visible = false;
                ToggleMode.Visible = false;
            }
            UpdateLayout();
        }

        private void ToggleParameters_Click(object sender, EventArgs e)
        {
            groupTriggerParameters.Visible = !groupTriggerParameters.Visible;
            groupBuildServers.Visible = !groupBuildServers.Visible;
            UpdateLayout();
        }

        private void UpdateLayout()
        {
            if (TriggerParameters.Visible)
            {
                OK.Top = 10 + groupBuildServers.Top + groupBuildServers.Height + groupTriggerParameters.Height;
                Cancel.Top = 10 + groupBuildServers.Top + groupBuildServers.Height + groupTriggerParameters.Height;
                ToggleMode.Top = 10 + groupBuildServers.Top + groupBuildServers.Height + groupTriggerParameters.Height;
                this.Height = 40 + ToggleMode.Top + ToggleMode.Height;
                ToggleMode.Text = "<< Simple Mode";
            }
            else
            {
                OK.Top = 10 + Data.Top + Data.Height;
                Cancel.Top = 10 + Data.Top + Data.Height;
                ToggleMode.Top = 10 + Data.Top + Data.Height;
                this.Height = 40 + ToggleMode.Top + ToggleMode.Height;
                ToggleMode.Text = "Advanced Mode >>";
            }

            if (ScheaduleOverview.Visible)
            {
                Data.Width -= ScheaduleOverview.Width + 10; 
            }
            else
            {
                Data.Width = Type.Width;
            }
        }

        private void PopulateBuildServers()
        {
            if (_trigger != null)
            {
                if (Type.Text == TriggerType.CheckinInclude.ToString())
                {
                    foreach (string server in _trigger.Servers)
                    {
                        BuildServers.Rows.Add(server);

                        //Enable Advanced mode
                        groupTriggerParameters.Visible = true;
                        groupBuildServers.Visible = true;
                    }
                }
            }
        }

        private void PopulateOverriddenParameters()
        {
            if (_trigger != null)
            {
                if (Type.Text == TriggerType.CheckinInclude.ToString())
                {
                    foreach (DataGridViewRow row in TriggerParameters.Rows)
                    {
                        if (_trigger.Parameters.ContainsKey((string)row.Cells[0].Value))
                        {
                            row.Cells[2].Value = _trigger.Parameters[(string)row.Cells[0].Value];

                            //Enable Advanced mode
                            groupTriggerParameters.Visible = true;
                            groupBuildServers.Visible = true;
                        }
                    }
                }
            }
        }
        
        private List<string> ExtractBuildServers()
        {
            List<string> result = new List<string>();

            if (Type.Text == TriggerType.CheckinInclude.ToString())
            {
                foreach (DataGridViewRow row in BuildServers.Rows)
                {
                    string value = (string)row.Cells[0].Value;
                    if (!String.IsNullOrEmpty(value))
                    {
                        result.Add(value);
                    }
                }
            }

            return result;
        }

        private Dictionary<string, string> ExtractOverriddenParameters()
        {
            Dictionary<string, string> result = new Dictionary<string, string>();

            if (Type.Text == TriggerType.CheckinInclude.ToString())
            {
                foreach (DataGridViewRow row in TriggerParameters.Rows)
                {
                    string value = (string)row.Cells[2].Value;
                    if (!String.IsNullOrEmpty(value))
                    {
                        result.Add((string)row.Cells[0].Value, (string)row.Cells[2].Value);
                    }
                }
            }

            return result;
        }

        private void ScheaduleOverview_Click(object sender, EventArgs e)
        {
            string message = "";

            foreach (BuildType bt in _buildTypeProxy.ListBuildTypes(_teamProject))
            {
                foreach (Trigger t in bt.Triggers.Values)
                {
                    if (t.Type == TriggerType.Schedule)
                    {
                        message += t.Data + " running build type " + bt.Name;
                    }
                }
            }

            MessageBox.Show(message, "Overview of scheduled build types for project " + _teamProject);
        }
    }
}