using System;
using System.Text;
using System.Collections.Generic;
using System.ServiceModel;
using System.Configuration;
using Microsoft.Build.Framework;
using Microsoft.Build.Utilities;
using Callista.BuildLab.Core;

namespace Callista.BuildLab.BuildTasks
{
    public class LoadOverriddenProperty : Task
    {
        private IBuildRepositoryManagement _buildRepositoryProxy = null;

        private string _uri;
        private string _name;
        private string _value;

        private Dictionary<string, string> _properties = null;

        [Required]
        public string BuildUri
        {
            get { return _uri; }
            set { _uri = value; }
        }

        [Required]
        public string PropertyToLoad
        {
            get { return _name; }
            set { _name = value; }
        }

        [Output]
        public string Value
        {
            get { return _value; }
        }

        public override bool Execute()
        {
            if (_properties == null)
            {
                try
                {
                    CreateAdminProxy();

                    if (_buildRepositoryProxy != null)
                    {
                        _properties = _buildRepositoryProxy.GetOverriddenParameters(BuildUri);
                    }
                }
                finally
                {
                    CloseAdminProxy();
                }
            }

            if (_properties.ContainsKey(PropertyToLoad))
            {
                _value = _properties[PropertyToLoad];
                return true;
            }
            else
            {
                _value = "";
                return false;
            }
        }

        #region AdminProxy Members

        private bool CreateAdminProxy()
        {
            string server = "";
            string path = "";

            try
            {
                NetTcpBinding binding = new NetTcpBinding(SecurityMode.None);

                binding.ReaderQuotas.MaxArrayLength = int.MaxValue;
                binding.ReaderQuotas.MaxBytesPerRead = int.MaxValue;
                binding.ReaderQuotas.MaxDepth = int.MaxValue;
                binding.ReaderQuotas.MaxStringContentLength = int.MaxValue;
                binding.ReaderQuotas.MaxBytesPerRead = int.MaxValue;

                binding.MaxReceivedMessageSize = int.MaxValue;
                binding.MaxBufferSize = int.MaxValue;

                binding.ReliableSession.Enabled = false;

                server = ConfigurationManager.OpenExeConfiguration(typeof(LoadOverriddenProperty).Assembly.Location).AppSettings.Settings["TFSBuildLab"].Value;
                path = string.Format("net.tcp://{0}/", server);

                ChannelFactory<IBuildRepositoryManagement> factoryBuildRepository = new ChannelFactory<IBuildRepositoryManagement>(binding, path);
                _buildRepositoryProxy = factoryBuildRepository.CreateChannel();
            }
            catch (Exception)
            {
                return false;
            }

            return true;
        }

        private void CloseAdminProxy()
        {
            if (_buildRepositoryProxy != null)
            {
                if (((ICommunicationObject)_buildRepositoryProxy).State == CommunicationState.Opened)
                {
                    ((ICommunicationObject)_buildRepositoryProxy).Close();
                }
                _buildRepositoryProxy = null;
            }
        }

        #endregion
    }
}

