using System;
using System.Text;
using System.Data;
using System.Data.SqlClient;
using System.Diagnostics;
using System.Security.Principal;
using System.Collections.Generic;
using Callista.BuildLab.Core.Model;

namespace Callista.BuildLab.Core.Data
{
    public interface IBuildLogRepository
    {
        List<LogEntry> ListBuildEvents(string teamProject, string buildType, string eventType, DateTime startDTM, DateTime endDTM);
    }

    public class BuildLogRepository : IBuildLogRepository
    {
        #region IBuildLogRepository Members

        List<LogEntry> IBuildLogRepository.ListBuildEvents(string teamProject, string buildType, string eventType, DateTime startDTM, DateTime endDTM)
        {
            List<LogEntry> leList = new List<LogEntry>();

            using (SqlConnection con = new SqlConnection(Common.GetConnectionString()))
            {
                con.Open();

                using (SqlCommand cmd = new SqlCommand("dbo.Log_List", con))
                {
                    cmd.Parameters.Add("@TeamProject", SqlDbType.NVarChar, 128);
                    cmd.Parameters.Add("@BuildType", SqlDbType.NVarChar, 128);
                    cmd.Parameters.Add("@EventType", SqlDbType.NVarChar, 32);
                    cmd.Parameters.Add("@StartDTM", SqlDbType.DateTime);
                    cmd.Parameters.Add("@EndDTM", SqlDbType.DateTime);

                    if (teamProject != string.Empty)
                    {
                        cmd.Parameters["@TeamProject"].Value = teamProject;
                    }
                    else
                    {
                        cmd.Parameters["@TeamProject"].Value = DBNull.Value;
                    }

                    if (buildType != string.Empty)
                    {
                        cmd.Parameters["@BuildType"].Value = buildType;
                    }
                    else
                    {
                        cmd.Parameters["@BuildType"].Value = DBNull.Value;
                    }

                    if (eventType != string.Empty)
                    {
                        cmd.Parameters["@EventType"].Value = eventType;
                    }
                    else
                    {
                        cmd.Parameters["@EventType"].Value = DBNull.Value;
                    }

                    cmd.Parameters["@StartDTM"].Value = startDTM;
                    cmd.Parameters["@EndDTM"].Value = endDTM;

                    cmd.CommandType = CommandType.StoredProcedure;

                    using (SqlDataReader rdr = cmd.ExecuteReader())
                    {
                        while (rdr.Read())
                        {
                            int logID = rdr.GetInt32(0);
                            DateTime eTime = rdr.GetDateTime(1);
                            string eType = rdr.GetString(2);
                            string projectName = rdr.GetString(3);
                            string buildTypeName = rdr.GetString(4);
                            string triggeredBy = rdr.IsDBNull(5) ? "" : rdr.GetString(5);
                            string buildUri = rdr.IsDBNull(6) ? "" : rdr.GetString(6);
                            string message = rdr.IsDBNull(7) ? "" : rdr.GetString(7);

                            leList.Add(new LogEntry(eTime, eType, projectName, buildTypeName, triggeredBy, buildUri, message));
                        }
                    }
                }
            }

            return leList;
        }

        #endregion
    }

    public static class BuildLog
    {
        public static void WriteEntryToLog(EventType eventType, string teamProjectName, string buildType, string triggeredBy)
        {
            BuildLog.WriteEntryToLog(eventType, teamProjectName, buildType, triggeredBy, "", "");
        }

        public static void WriteEntryToLog(EventType eventType, string teamProjectName, string buildType, string triggeredBy, string buildUri)
        {
            BuildLog.WriteEntryToLog(eventType, teamProjectName, buildType, triggeredBy, buildUri, "");
        }

        public static void WriteEntryToLog(EventType eventType, string teamProjectName, string buildType, string triggeredBy, string buildUri, string message)
        {
            if (!string.IsNullOrEmpty(message))
            {
                Trace.TraceError("{0} An error occurred in {1} for {2} the casue was: {3}", DateTime.Now, teamProjectName, buildType, message);
            }

            if (triggeredBy == String.Empty)
            {
                triggeredBy = WindowsIdentity.GetCurrent().Name;
            }

            using (SqlConnection con = new SqlConnection(Common.GetConnectionString()))
            {
                con.Open();

                using (SqlCommand cmd = new SqlCommand("dbo.Log_WriteEntry", con))
                {
                    cmd.Parameters.Add("@EventType", SqlDbType.NVarChar, 32);
                    cmd.Parameters.Add("@TeamProject", SqlDbType.NVarChar, 128);
                    cmd.Parameters.Add("@BuildType", SqlDbType.NVarChar, 128);
                    cmd.Parameters.Add("@TriggeredBy", SqlDbType.NVarChar, 512);
                    cmd.Parameters.Add("@BuildUri", SqlDbType.NVarChar, 512);
                    cmd.Parameters.Add("@Message", SqlDbType.NText);

                    cmd.Parameters["@EventType"].Value = eventType;
                    cmd.Parameters["@TeamProject"].Value = teamProjectName;
                    cmd.Parameters["@BuildType"].Value = buildType;
                    cmd.Parameters["@TriggeredBy"].Value = triggeredBy;

                    if (buildUri == String.Empty)
                    {
                        cmd.Parameters["@BuildUri"].Value = DBNull.Value;
                    }
                    else
                    {
                        cmd.Parameters["@BuildUri"].Value = buildUri;
                    }

                    if (message == String.Empty)
                    {
                        cmd.Parameters["@Message"].Value = DBNull.Value;
                    }
                    else
                    {
                        cmd.Parameters["@Message"].Value = message;
                    }

                    cmd.CommandType = CommandType.StoredProcedure;
                    cmd.ExecuteNonQuery();
                }
            }
        }
    }
}
