using System;
using System.IO;
using System.Text;
using System.Xml;
using System.Xml.Serialization;
using System.Collections.Generic;
using Microsoft.TeamFoundation.VersionControl.Client;

namespace Callista.BuildLab.Core.Model
{
    public class Changeset
    {
        private int _id;
        private string _comitter;
        private List<VersionedItem> _versionedItemList = new List<VersionedItem>();

        public int Id
        {
            get { return _id; }
            set { _id = value; }
        }

        public string Comitter
        {
            get { return _comitter; }
            set { _comitter = value; }
        }

        public List<VersionedItem> VersionedItems
        {
            get { return _versionedItemList; }
            set { _versionedItemList = value; }
        }

        public static Changeset CreateFromObject(Microsoft.TeamFoundation.VersionControl.Client.Changeset changesetObject)
        {
            Changeset cs = new Changeset();

            cs.Id = changesetObject.ChangesetId;
            cs.Comitter = changesetObject.Committer;

            for (int i = 0; i < changesetObject.Changes.Length; i++)
            {
                if (changesetObject.Changes[i].Item.ItemType == ItemType.File)
                {
                    VersionedItem vi = new VersionedItem();
                    int position = changesetObject.Changes[i].Item.ServerItem.LastIndexOf('/')+1;
                    vi.Name = changesetObject.Changes[i].Item.ServerItem.Substring(position);
                    vi.Folder = changesetObject.Changes[i].Item.ServerItem.Substring(0,position);
                    vi.FullPath = changesetObject.Changes[i].Item.ServerItem;
                    cs.VersionedItems.Add(vi);
                }
            }
            
            return cs;
        }

        public static Changeset CreateFromEvent(CheckinEvent checkinEvent)
        {
            Changeset cs = new Changeset();
            
            cs.Id = checkinEvent.Number;
            cs.Comitter = checkinEvent.Committer;

            foreach (ClientArtifact a in checkinEvent.Artifacts)
            {
                if (a.ArtifactType == "VersionedItem" && a.ChangeType != "merge")
                {
                    VersionedItem vi = new VersionedItem();
                    vi.Name = a.Item;
                    vi.Folder = a.Folder;
                    vi.FullPath = a.ServerItem;
                    cs.VersionedItems.Add(vi);
                }
            }

            return cs;
        }
    }
}
