using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.TeamFoundation.Client;
using Microsoft.TeamFoundation.Server;
using System.Configuration;

namespace Callista.BuildLab.Core
{
    public class SubscriptionService
    {
        private TeamFoundationServer _tfs;
        private IEventService _eventService;
        private Dictionary<int, EventType> _subscriptions;
        private string _userId;

        public enum EventType
        {
            NotUsed,
            CheckinEvent,
            BuildCompletionEvent,
            BuildStatusChangeEvent,
            WorkItemChangedEvent
        }

        public SubscriptionService(TeamFoundationServer tfs)
        {
            _tfs = tfs;
            _eventService = (IEventService)tfs.GetService(typeof(IEventService));
            _subscriptions = new Dictionary<int, EventType>();
            if (!String.IsNullOrEmpty(ConfigurationManager.AppSettings["tfsUser"]))
            {
                _userId = ConfigurationManager.AppSettings["tfsUser"];
            }
            else
            {
                _userId = System.Security.Principal.WindowsIdentity.GetCurrent().Name;
            }
        }

        private DeliveryPreference GetDeliveryPreference(string address)
        {
            DeliveryPreference dp = new DeliveryPreference();
            dp.Address = address;
            dp.Schedule = DeliverySchedule.Immediate;
            dp.Type = DeliveryType.Soap;
            return dp;
        }

        public void UpdateSubscriptions()
        {
            Subscription[] subscriptions = _eventService.EventSubscriptions(_userId);

            // Get address from WCF host for notifications
            string buildLabAddress = BuildManager.NotificationHost.Description.Endpoints[0].Address.Uri.AbsoluteUri;

            int id;
            id = UpdateSubscription(subscriptions, EventType.CheckinEvent, buildLabAddress);
            _subscriptions.Add(id, EventType.CheckinEvent);
            id = UpdateSubscription(subscriptions, EventType.BuildCompletionEvent, buildLabAddress);
            _subscriptions.Add(id, EventType.BuildCompletionEvent);
            id = UpdateSubscription(subscriptions, EventType.BuildStatusChangeEvent, buildLabAddress);
            _subscriptions.Add(id, EventType.BuildStatusChangeEvent);
            id = UpdateSubscription(subscriptions, EventType.WorkItemChangedEvent, buildLabAddress);
            _subscriptions.Add(id, EventType.WorkItemChangedEvent);

        }

        public void DeleteSubscriptions()
        {
            foreach(int id in _subscriptions.Keys)
            {
                Unsubscribe(id);
            }
        }

        public EventType GetEventType(int subscriptionId)
        {
            EventType et = EventType.NotUsed;
            _subscriptions.TryGetValue(subscriptionId, out et);
            return et;
        }

        private int UpdateSubscription(Subscription[] subscriptions, EventType type, string address)
        {
            string tag = string.Format("TFSBuildLab:{0}", type.ToString());
            foreach (Subscription s in subscriptions)
            {
                if (s.Tag == tag)
                {
                    if (s.DeliveryPreference.Address == address)
                        return s.ID;
                    else
                        Unsubscribe(s.ID);
                }
            }
            return Subscribe(type.ToString(), tag, address);
        }

        private int Subscribe(string eventType, string tag, string address)
        {
            return _eventService.SubscribeEvent(_userId, eventType, "", GetDeliveryPreference(address), tag);
        }

        private void Unsubscribe(int subscriptionId)
        {
            _eventService.UnsubscribeEvent(subscriptionId);
        }
    }
}
