using System;
using System.Text;
using System.Reflection;
using System.Configuration;
using System.Net;
using System.Diagnostics;
using System.Collections;
using System.Collections.Generic;
using Microsoft.TeamFoundation.Server;
using Microsoft.TeamFoundation.Client;
using Microsoft.TeamFoundation.WorkItemTracking.Client;

using Callista.BuildLab.Core.Data;
using Callista.BuildLab.Core.Model;

namespace Callista.BuildLab.Core
{
    public class TFS
    {
        #region TFS Services
        
        private static TeamFoundationServer _tfs;
        private static TeamFoundationServer ServerInstance
        {
            get 
            {
                if (_tfs == null)
                {
                    string tfsUri = ConfigurationManager.AppSettings["tfsServerAddress"];
                    string tfsUser = ConfigurationManager.AppSettings["tfsUser"];
                    string tfsPassword = ConfigurationManager.AppSettings["tfsPassword"];

                    if (string.IsNullOrEmpty(tfsUser) && string.IsNullOrEmpty(tfsPassword))
                    {
                        _tfs = new TeamFoundationServer(tfsUri);
                    }
                    else
                    {
                        // check if useraccount contains a domain part, if so split and user overloaded c'tor
                        if (tfsUser.Contains("\\"))
                        {
                            string[] userInfo = tfsUser.Split('\\');
                            _tfs = new TeamFoundationServer(tfsUri, new NetworkCredential(userInfo[1], tfsPassword, userInfo[0]));
                        }
                        else
                        {
                            _tfs = new TeamFoundationServer(tfsUri, new NetworkCredential(tfsUser, tfsPassword));
                        }
                    }
                }
                return _tfs; 
            }
        }

        private static WorkItemStore _wit;
        public static WorkItemStore WIT
        {
            get
            {
                if (_wit == null)
                {
                    _wit = (WorkItemStore)ServerInstance.GetService(typeof(WorkItemStore));
                }
                return _wit;
            }
        }

        private static ICommonStructureService _css;
        public static ICommonStructureService CSS
        {
            get
            {
                if (_css == null)
                {
                    _css = (ICommonStructureService)ServerInstance.GetService(typeof(ICommonStructureService));
                }
                return _css;
            }
        }

        private static IGroupSecurityService _gss;
        public static IGroupSecurityService GSS
        {
            get
            {
                if (_gss == null)
                {
                    _gss = (IGroupSecurityService)ServerInstance.GetService(typeof(IGroupSecurityService));
                }
                return _gss;
            }
        }

        private static BuildService _buildService = null;
        public static BuildService BuildService
        {
            get
            {
                if (_buildService == null)
                {
                    _buildService = new BuildService(ServerInstance);
                }
                return _buildService;
            }
        }

        private static SubscriptionService _subscriptionService = null;
        public static SubscriptionService SubscriptionService
        {
            get
            {
                if (_subscriptionService == null)
                {
                    _subscriptionService = new SubscriptionService(ServerInstance);
                }
                return _subscriptionService;
            }
        }

        #endregion

        public static void ResetAllCaches()
        {
            _cachedTeamProjects.Clear();
            BuildService.ResetAllCaches();
        }

        static List<TeamProject> _cachedTeamProjects = new List<TeamProject>();
        public static List<TeamProject> ListTeamProjects()
        {
            lock (_cachedTeamProjects)
            {
                if (_cachedTeamProjects.Count == 0)
                {
                    foreach (ProjectInfo pi in CSS.ListAllProjects())
                    {
                        TeamProject teamProject = new TeamProject(pi.Name);
                        foreach (BuildType bt in BuildService.ListBuildTypes(pi.Name))
                        {
                            teamProject.BuildTypes.Add(bt.Name, bt);
                        }
                        _cachedTeamProjects.Add(teamProject);
                    }
                }
                return _cachedTeamProjects;
            }
        }

        public static void CreateWorkItem(string project, string type, string title, string description, Dictionary<string, string> fields)
        {
            WorkItemTypeCollection witList = WIT.Projects[project].WorkItemTypes;
            WorkItemType wit = witList[type];

            WorkItem workItem = new WorkItem(wit);
            workItem.Title = title;
            workItem.Description = description;

            foreach (string fieldname in fields.Keys)
            {
                if (workItem.Fields.Contains(fieldname))
                {
                    workItem.Fields[fieldname].Value = fields[fieldname];
                }
            }

            if (workItem.Validate().Count == 0)
            {
                workItem.Save();
            }
        }

        public static Identity GetUserInfo(string accountName)
        {
            return GSS.ReadIdentity(SearchFactor.AccountName, accountName, QueryMembership.None);
        }
    }
}
