using System;
using System.IO;
using System.Data;
using System.Data.SqlClient;
using System.ComponentModel;
using System.Windows.Forms;
using System.Collections;
using System.Collections.Specialized;
using System.Configuration.Install;
using System.Reflection;
using System.Text;
using System.Text.RegularExpressions;

namespace TFSBuildLab.CreateDatabase
{
    [RunInstaller(true)]
    public partial class CreateDatabase : Installer
    {
        public CreateDatabase()
        {
            InitializeComponent();
        }

        public override void Install(IDictionary stateSaver)
        {
            base.Install(stateSaver);

            StringDictionary parameters = Context.Parameters;
            ExecuteCreateDatabase(parameters["dbserver"], parameters["dbname"], parameters["dbaccount"]);
        }

        private string GetSqlScript(string name)
        {
            Stream resource = Assembly.GetExecutingAssembly().GetManifestResourceStream(Assembly.GetExecutingAssembly().GetName().Name + "." + name);
            StreamReader reader = new StreamReader(resource, Encoding.Unicode);
            return reader.ReadToEnd();
        }

        private void ExecuteCreateDatabase(string server, string name, string login)
        {
            using (SqlConnection con = new SqlConnection("Integrated Security=SSPI;Persist Security Info=False;Data Source=" + server))
            {
                con.Open();
                con.ChangeDatabase("master");

                using (SqlCommand cmd = new SqlCommand("CREATE DATABASE " + name, con))
                {
                    try
                    {
                        cmd.CommandType = CommandType.Text;
                        cmd.ExecuteNonQuery();

                        //If db doesn't exists we execute this otherwise we fail silently
                        ExecuteCreateTablesAndData(server, name, login);
                    }
                    catch (SqlException ex)
                    {
                        //Supress fault that 
                        //db already exists!

                        if (ex.Number != 1801)
                        {
                            MessageBox.Show("Could not create database please revert to manually install the database!" + Environment.NewLine + Environment.NewLine + "The error occuring was: " + ex.ToString(), "Unable to install database for TfsBuildLab", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                        }
                    }
                }
            }
        }

        private void ExecuteCreateTablesAndData(string server, string name, string login)
        {
            using (SqlConnection con = new SqlConnection("Integrated Security=SSPI;Persist Security Info=False;Data Source=" + server))
            {
                con.Open();
                con.ChangeDatabase(name);

                string sql = GetSqlScript("CreateDatabase.sql");
                string[] commands = Regex.Split(sql, @"\s[Gg][Oo]\s");

                foreach (string command in commands)
                {
                    string temp = command.Replace("\r"," ");

                    using (SqlCommand cmd = new SqlCommand(temp, con))
                    {
                        cmd.CommandType = CommandType.Text;
                        cmd.ExecuteNonQuery();
                    }
                }

                sql = GetSqlScript("CreateLogin.sql");
                commands = Regex.Split(sql, @"\s[Gg][Oo]\s");

                foreach (string command in commands)
                {
                    string temp = command.Replace(@"DOMAIN\ACCOUNT", login);

                    using (SqlCommand cmd = new SqlCommand(temp, con))
                    {
                        cmd.CommandType = CommandType.Text;
                        cmd.ExecuteNonQuery();
                    }
                }

                sql = GetSqlScript("InsertData.sql");

                using (SqlCommand cmd = new SqlCommand(sql, con))
                {
                    cmd.CommandType = CommandType.Text;
                    cmd.ExecuteNonQuery();
                }
            }
        }
    }
}