using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Configuration;
using System.IO;
using System.Xml;
using System.Reflection;
using Callista.BuildLab.Core.Model;

namespace Callista.BuildLab.NotifyClient
{
    public partial class AddNotification : Form
    {
        private Notification _createdNotification = null;
        private List<TeamProject> _projects = null;
        
        public Notification CreatedNotification
        {
            get { return _createdNotification; }
            set { _createdNotification = value; }
        }

        public AddNotification(List<TeamProject> projects)
        {
            _projects = projects;
            InitializeComponent();
        }

        private void comboBox1_SelectedIndexChanged(object sender, EventArgs e)
        {
            switch ((NotificationType)Enum.Parse(typeof(NotificationType), NotificationTypeList.Text, true))
            {
                case NotificationType.Checkin:
                    label2.Text = " is made in ";
                    label3.Text = " for the following source control path: ";
                    NotificationProject.Left = label2.Right + 10;
                    label3.Left = NotificationProject.Right + 10;
                    NotificationDataEdit.Top = NotificationProject.Top;
                    NotificationDataEdit.Left = label3.Right + 10;
                    NotificationDataEdit.Width = Cancel.Right - NotificationDataEdit.Left;
                    NotificationDataEdit.Visible = true;
                    NotificationDataCombobox.Visible = false;
                    break;
                case NotificationType.Build:
                    label2.Text = " completes for ";
                    label3.Text = " and the following build type: ";
                    NotificationProject.Left = label2.Right + 10;
                    label3.Left = NotificationProject.Right + 10;
                    NotificationDataCombobox.Top = NotificationProject.Top;
                    NotificationDataCombobox.Left = label3.Right + 10;
                    NotificationDataCombobox.Width = Cancel.Right - NotificationDataCombobox.Left;
                    NotificationDataCombobox.Visible = true;
                    NotificationDataEdit.Visible = false;
                    break;
            }
        }

        private void NotificationProject_SelectedIndexChanged(object sender, EventArgs e)
        {
            TeamProject project = (TeamProject)NotificationProject.SelectedValue;
            List<BuildType> buildTypes = new List<BuildType>();

            foreach (BuildType bt in project.BuildTypes.Values)
            {
                buildTypes.Add(bt);
            }

            NotificationDataCombobox.SuspendLayout();
            NotificationDataCombobox.DataSource = null;
            NotificationDataCombobox.DataSource = buildTypes;
            NotificationDataCombobox.DisplayMember = "Name";
            NotificationDataCombobox.ResumeLayout();
        }

        private void AddNotification_Load(object sender, EventArgs e)
        {
            List<TeamProject> projects = new List<TeamProject>();

            foreach (TeamProject tp in _projects)
            {
                if (tp.BuildTypes.Count > 0)
                {
                    projects.Add(tp);
                }
            }

            NotificationTypeList.SelectedIndex = 0;
            NotificationProject.DataSource = projects;
            NotificationProject.DisplayMember = "Name";
        }

        private void Cancel_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.Cancel;
            this.Close();
        }

        private void OK_Click(object sender, EventArgs e)
        {
            NotificationType type = (NotificationType)Enum.Parse(typeof(NotificationType), NotificationTypeList.Text, true);
            string project = NotificationProject.Text;
            string data = "";
            
            if (type == NotificationType.Checkin)
            {
                data = NotificationDataEdit.Text;
            }
            else
            {
                data = NotificationDataCombobox.Text;
            }

            _createdNotification = new Notification(Guid.NewGuid().ToString(), project, type, data);
            
            this.DialogResult = DialogResult.OK;
            this.Close();
        }
    }
}