using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Reflection;
using System.Xml;
using System.Xml.Serialization;
using System.IO;
using Callista.BuildLab.Core.Model;

namespace Callista.BuildLab.NotifyClient
{
    public partial class ConfigForm : Form
    {
        private List<TeamProject> _projects = null;
        
        public ConfigForm(List<TeamProject> projects)
        {
            _projects = projects;
            InitializeComponent();
        }

        private void ConfigForm_SizeChanged(object sender, EventArgs e)
        {
            if (this.WindowState == FormWindowState.Minimized)
            {
                this.Close();
            }
        }

        private void ConfigForm_Load(object sender, EventArgs e)
        {
            NotificationGridView.Columns.Add("ID", "ID");
            NotificationGridView.Columns.Add("Project", "Project");
            NotificationGridView.Columns.Add("Type", "Type");
            NotificationGridView.Columns.Add("Data", "Data");
            NotificationGridView.Columns["ID"].DataPropertyName = "ID";
            NotificationGridView.Columns["ID"].Visible = false;
            NotificationGridView.Columns["Project"].DataPropertyName = "Project";
            NotificationGridView.Columns["Type"].DataPropertyName = "Type";
            NotificationGridView.Columns["Data"].DataPropertyName = "Data";

            NotificationGridView.AutoGenerateColumns = false;

            RefreshNotifications();
        }

        private void RefreshNotifications()
        {
            NotificationGridView.SuspendLayout();
            NotificationGridView.DataSource = null;
            NotificationGridView.DataSource = Program.RegisteredNotifications;
            NotificationGridView.ResumeLayout();
        }

        private void AddNotification_Click(object sender, EventArgs e)
        {
            AddNotification dlg = new AddNotification(_projects);
            if (dlg.ShowDialog() == DialogResult.OK)
            {
                Program.RegisteredNotifications.Add(dlg.CreatedNotification);
                RefreshNotifications();
            }
        }

        private void DeleteNotification_Click(object sender, EventArgs e)
        {
            foreach (Notification n in Program.RegisteredNotifications)
            {
                if (n.ID.Equals(NotificationGridView.SelectedRows[0].Cells[0].Value))
                {
                    Program.RegisteredNotifications.Remove(n);
                    break;
                }
            }

            RefreshNotifications();
            DeleteNotification.Enabled = false;
        }

        private void NotificationGridView_CellClick(object sender, DataGridViewCellEventArgs e)
        {
            DeleteNotification.Enabled = true;
        }
    }
}