using System;
using System.Windows.Forms;
using System.Collections.Generic;
using Microsoft.TeamFoundation.VersionControl.Client;

namespace TfsBuildLab.Policies.RequiresComment
{
    [Serializable]
    public class RequiresCommentPolicy : PolicyBase
    {
        private const string instructions = "Deploy the assembly to your machine and create the following entry in the registry \n\n" +
                                            "\t Key: [HKEY_LOCAL_MACHINE\\SOFTWARE\\Microsoft\\VisualStudio\\8.0\\TeamFoundation\\SourceControl\\Checkin Policies] \n" +
                                            "\t Name: name of assembly (without extension) \n" +
                                            "\t Value: full path to assembly";

	    public override string Description
	    {
		    get { return "Remind users to add meaningful comments to their checkins"; }
	    }

        public override string Type
	    {
		    get { return "TfsBuildLab Requires Comment Policy"; }
	    }
        
        public override string TypeDescription
	    {
		    get { return "This policy will prompt the user to include a comment to the checked-in files."; }
	    }

        public override string InstallationInstructions
	    {
		    get { return instructions; }
	    }

        public override bool CanEdit
        {
            get { return false; }
        }

        public override void Activate(PolicyFailure failure)
        {
            MessageBox.Show("Please provide comments for your checkin.", "How to fix your policy failure");
        }

        public override bool Edit(IPolicyEditArgs policyEditArgs)
        {
		    return true;
	    }

        public override void DisplayHelp(PolicyFailure failure)
        {
            MessageBox.Show("This policy requires comments for all checkins.", "Prompt Policy Help");
        }

        public override PolicyFailure[] Evaluate()
        {
            string proposedComment = PendingCheckin.PendingChanges.Comment;
            if (String.IsNullOrEmpty(proposedComment))
            {
                return new PolicyFailure[] { new PolicyFailure("Please provide some comments about your checkin.", this) };
            }
            else
            {
                return new PolicyFailure[] {};
            }
        }
    }
}