using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace Callista.BuildLab.Policies.RestrictedPaths
{
    public partial class RestrictedPathsForm : Form
    {
        private List<CheckinRestriction> _restrictions = new List<CheckinRestriction>();

        public List<CheckinRestriction> Restrictions
	    {
            get { return _restrictions; }
            set { _restrictions = value; }
	    }
	
        public RestrictedPathsForm()
        {
            InitializeComponent();
        }

        private void RestrictedPathsForm_Load(object sender, EventArgs e)
        {
            foreach (CheckinRestriction cr in _restrictions)
            {
                DataGridViewRow row = new DataGridViewRow();
                DataGridViewTextBoxCell cellPath = new DataGridViewTextBoxCell();
                DataGridViewTextBoxCell cellName = new DataGridViewTextBoxCell();
                DataGridViewTextBoxCell cellComment = new DataGridViewTextBoxCell();
                DataGridViewCheckBoxCell cellBreak = new DataGridViewCheckBoxCell();

                cellPath.Value = cr.Path;
                cellName.Value = cr.Name;
                cellComment.Value = cr.Comment;
                cellBreak.Value = cr.BreakOnFirstViolation;

                row.Cells.Add(cellPath);
                row.Cells.Add(cellName);
                row.Cells.Add(cellComment);
                row.Cells.Add(cellBreak);

                PathsGridView.Rows.Add(row);
            }
        }

        private void OK_Click(object sender, EventArgs e)
        {
            _restrictions.Clear();

            foreach (DataGridViewRow row in PathsGridView.Rows)
            {
                if (!string.IsNullOrEmpty((string)row.Cells[0].Value))
                {
                    string path = (string)row.Cells[0].Value;
                    string name = (string)row.Cells[1].Value;
                    string comment = (string)row.Cells[2].Value;
                    bool breakOnFirst = (bool)row.Cells[3].Value;

                    _restrictions.Add(new CheckinRestriction(path, name, comment, breakOnFirst));
                }
            }

            this.DialogResult = DialogResult.OK;
            this.Close();
        }

        private void Cancel_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.Cancel;
            this.Close();
        }
    }
}