using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.ServiceProcess;
using System.Text;
using Callista.BuildLab.Core;

namespace Callista.BuildLab.Service
{
    public partial class ServiceMain : ServiceBase
    {
        private BuildManager bm = new BuildManager();

        public ServiceMain()
        {
            InitializeComponent();
        }

        private void WriteToEventLog(string message)
        {
            EventLog.WriteEntry("TfsBuildlab", String.Format("{0} {1}", DateTime.Now, message), EventLogEntryType.Information);
        }

        private void WriteToEventLog(string message, Exception ex)
        {
            EventLog.WriteEntry("TfsBuildlab", String.Format("{0} {1}: An exception occured ... {2}", DateTime.Now, message, ex.ToString()), EventLogEntryType.Error);
        }

        protected override void OnStart(string[] args)
        {
            try
            {
                bm.Start();
                WriteToEventLog("TFSBuildLab service started");
            }
            catch (Exception ex)
            {
                WriteToEventLog("TFSBuildLab service failed to start", ex);
            }
        }

        protected override void OnStop()
        {
            try
            {
                bm.Stop();
                WriteToEventLog("TFSBuildLab service stopped");
            }
            catch (Exception ex)
            {
                WriteToEventLog("TFSBuildLab service failed to stop", ex);
            }
        }

        protected override void OnPause()
        {
            try
            {
                bm.Pause();
                WriteToEventLog("TFSBuildLab service paused");
            }
            catch (Exception ex)
            {
                WriteToEventLog("TFSBuildLab service failed to pause", ex);
            }
        }

        protected override void OnContinue()
        {
            try
            {
                bm.Resume();
                WriteToEventLog("TFSBuildLab service resumed");
            }
            catch (Exception ex)
            {
                WriteToEventLog("TFSBuildLab service failed to resume", ex);
            }
        }

        protected override void OnShutdown()
        {
            try
            {
                bm.Stop();
                WriteToEventLog("TFSBuildLab service shutdown");
            }
            catch (Exception ex)
            {
                WriteToEventLog("TFSBuildLab service failed to shutdown", ex);
            }
        }
    }
}